package cerebral;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.ProgressMonitor;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.metal.MetalButtonUI;

import prefuse.util.ColorLib;
import prefuse.util.StrokeLib;
import cerebral.colors.ColorLegend;
import cerebral.colors.ColorRange;
import cerebral.colors.ColorScaleBar;
import cerebral.colors.ColorScaleDialog;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;
import cytoscape.view.CytoscapeDesktop;

public class ControlPanel extends JPanel implements PropertyChangeListener,
        ListSelectionListener, MouseListener {

    private static final long serialVersionUID = 1L;

    private Map displays = new HashMap();

    private float beta;

    private float labelDensity = 15 / 4000f;;

    private float groupLabelSize;

    private boolean showHulls;

    private boolean showSeparators;

    private boolean highQuality;

    private boolean isRatioData;

    private Cerebral currentVis;

    private JTable groupsTable;

    private JButton addGroup;

    private JButton deleteGroup;

    private JButton relayout;

    private Color hullColor;

    private ColorLegend expressionColorLegend;

    private ColorLegend comparisonColorLegend;

    private ColorLegend updatingLegend;

    private ColorScaleDialog csd;

    JCheckBox    checkRatioData;
    
    private static String PLUS_MINUS = Character.toString((char) 177);

    private float[] dashes = { 1, 3 };

    private float sigCutoff = 0.05f;

    private JSlider sigCutoffFilter;

    JFormattedTextField sigCutoffVal;

    private float foldChangeCutoff = 1.5f;

    private JSlider foldChangeFilter;

    JFormattedTextField foldChangeVal;

    private static ControlPanel controlPanel = new ControlPanel();

    class CirclePanel extends JPanel {
        private static final long serialVersionUID = 1L;

        private Color fill;

        private Color border;

        public CirclePanel(Color fill) {
            this.border = Color.black;
            this.fill = fill;
            setPreferredSize(new Dimension(10, 10));
        }

        public CirclePanel(Color fill, Color border) {
            this.border = border;
            this.fill = fill;
        }

        protected void paintComponent(Graphics gIn) {
            super.paintComponent(gIn);
            Graphics2D g = (Graphics2D) gIn;
            g.setColor(fill);
            g.fillOval(0, 0, getWidth() - 1, getHeight() - 1);
            g.setColor(border);
            g.drawOval(0, 0, getWidth() - 1, getHeight() - 1);

        }
    }

    class SquarePanel extends JPanel {
        private static final long serialVersionUID = 1L;

        private Color fill;

        public SquarePanel(Color fill) {
            this.fill = fill;
            setPreferredSize(new Dimension(10, 10));
        }

        protected void paintComponent(Graphics gIn) {
            super.paintComponent(gIn);
            Graphics2D g = (Graphics2D) gIn;
            g.setColor(fill);
            g.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
            g.setColor(Color.black);
            g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);

        }
    }

    class LinePanel extends JPanel {

        private static final long serialVersionUID = 1L;

        private Stroke line;

        public LinePanel(Stroke line) {
            this.line = line;
            setPreferredSize(new Dimension(20, 10));
        }

        protected void paintComponent(Graphics gIn) {
            super.paintComponent(gIn);
            Graphics2D g = (Graphics2D) gIn;
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g.setColor(Color.black);
            g.setStroke(line);
            g.drawLine(0, 0, getWidth() - 1, getHeight() - 1);
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_OFF);
        }
    }

    class ArrowPanel extends JPanel {

        private static final long serialVersionUID = 1L;

        private int arrowHead;

        public ArrowPanel(int arrowHead) {
            this.arrowHead = arrowHead;
            setPreferredSize(new Dimension(20, 10));
            setMinimumSize(new Dimension(20, 10));
        }

        protected void paintComponent(Graphics gIn) {
            super.paintComponent(gIn);
            Graphics2D g = (Graphics2D) gIn;
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g.setColor(Color.black);
            g.setStroke(StrokeLib.getStroke(1.0f));
            g.drawLine(0, getHeight() / 2, getWidth() - 10, getHeight() / 2);
            if (arrowHead == 0) {
                g.fillOval(getWidth() - 11, 1, 8, getHeight() - 2);
            } else {
                g
                        .fillPolygon(getDiamond(getWidth() - 6, 1, 10,
                                getHeight() - 1));
            }

            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_OFF);
        }

        private Polygon getDiamond(int x, int y, int w, int h) {
            Polygon diamond = new Polygon();
            diamond.addPoint(x, y + h);
            diamond.addPoint(x - w / 2, y + h / 2);
            diamond.addPoint(x, y);
            diamond.addPoint(x + w / 2, y + h / 2);
            diamond.addPoint(x, y + h);
            return diamond;
        }

    }

    public static ControlPanel instance() {
        return controlPanel;
    }

    public void addDisplay(Cerebral vis, String identifier) {
        displays.put(identifier, vis);
    }

    public void removeDisplay(String identifier) {
        displays.remove(identifier);
    }

    public Cerebral getCurrentVis() {
        return currentVis;
    }

    private ControlPanel() {
        Cytoscape.getDesktop().getSwingPropertyChangeSupport()
                .addPropertyChangeListener(this);

        setLayout(new GridBagLayout());
        setAlignmentX(Component.LEFT_ALIGNMENT);
        groupsTable = new JTable();
        groupsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        groupsTable.getSelectionModel().addListSelectionListener(this);
        groupsTable.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                int row = groupsTable.rowAtPoint(e.getPoint());
                if (row != -1) {
                    if (currentVis != null) {
                        currentVis.groupsManager.setCurrentGroup(row, e
                                .isShiftDown());
                    }
                }
            }
        });

        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = 2;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.LINE_START;
        beta = 1.0f;
        JLabel edgeControllerLabel = new JLabel("Edge curviness", JLabel.CENTER);

        JSlider edgeController = new JSlider(JSlider.HORIZONTAL, 0, 10, 10);
        Hashtable labelTable = new Hashtable();

        labelTable.put(new Integer(0), new JLabel("Straight"));
        labelTable.put(new Integer(10), new JLabel("Bundled"));
        edgeController.setLabelTable(labelTable);
        edgeController.setPaintLabels(true);
        edgeController.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                beta = source.getValue() * 0.1f;
                Iterator displaysIt = displays.values().iterator();
                while (displaysIt.hasNext()) {
                    Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                    cytoPrefuse.edgeBundler.setBeta(beta);
                    cytoPrefuse.getVisualization().run("color");
                    cytoPrefuse.getVisualization().run("repaint");

                }
            }

        });

        edgeController.setAlignmentX(Component.LEFT_ALIGNMENT);
        c.gridwidth = 2;
        c.gridy = 12;
        add(edgeControllerLabel, c);
        c.gridy = 13;
        add(edgeController, c);

        JLabel labelDensityLabel = new JLabel("Label density", JLabel.CENTER);

        JSlider labelDensityController = new JSlider(JSlider.HORIZONTAL, 0,
                100, 15);
        labelTable = new Hashtable();
        labelTable.put(new Integer(0), new JLabel("Dense"));
        labelTable.put(new Integer(100), new JLabel("Sparse"));
        labelDensityController.setLabelTable(labelTable);
        labelDensityController.setPaintLabels(true);
        labelDensityController.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                labelDensity = source.getValue() / 4000f;
                Iterator displaysIt = displays.values().iterator();
                while (displaysIt.hasNext()) {
                    Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                    cytoPrefuse.labelLayout.setExtraSpace(labelDensity);
                }
            }

        });
        labelDensityController.setAlignmentX(Component.LEFT_ALIGNMENT);
        add(Box.createRigidArea(new Dimension(0, 20)));

        c.gridy = 14;
        add(labelDensityLabel, c);
        c.gridy = 15;
        add(labelDensityController, c);

        groupLabelSize = 1.0f;
        JLabel labelDownstreamLabelSize = new JLabel("Group label size",
                JLabel.CENTER);

        JSlider downstreamLabelSizeSlider = new JSlider(JSlider.HORIZONTAL, 0,
                100, 40);
        labelTable = new Hashtable();
        labelTable.put(new Integer(0), new JLabel("Off"));
        labelTable.put(new Integer(20), new JLabel("Small"));
        labelTable.put(new Integer(100), new JLabel("Large"));
        downstreamLabelSizeSlider.setLabelTable(labelTable);
        downstreamLabelSizeSlider.setPaintLabels(true);
        downstreamLabelSizeSlider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                if (source.getValue() >= 20) {
                    groupLabelSize = source.getValue() / 30.0f;
                } else {
                    groupLabelSize = 0;
                }
                Iterator displaysIt = displays.values().iterator();
                while (displaysIt.hasNext()) {
                    Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                    cytoPrefuse.labelLayout
                            .setAggregateLabelScale(groupLabelSize);
                }
            }

        });
        downstreamLabelSizeSlider.setAlignmentX(Component.LEFT_ALIGNMENT);

        c.gridy = 18;
        add(labelDownstreamLabelSize, c);
        c.gridy = 19;
        add(downstreamLabelSizeSlider, c);

        JCheckBox checkShowHulls = new JCheckBox("Show function clusters");
        checkShowHulls.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                showHulls = e.getStateChange() == ItemEvent.SELECTED;
                Iterator displaysIt = displays.values().iterator();
                while (displaysIt.hasNext()) {
                    Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                    cytoPrefuse.setShowHulls(showHulls);
                }
            }
        });
        checkShowHulls.setSelected(true);
        checkShowHulls.setAlignmentX(Component.LEFT_ALIGNMENT);

        c.gridy = 20;
        c.gridx = 0;
        c.weightx = 0.5;
        c.gridwidth = 1;
        c.fill = GridBagConstraints.NONE;

        add(checkShowHulls, c);

        // JLabel hullColorLabel = new JLabel("Cluster color");
        final JButton hullColorButton = new JButton("");
        hullColorButton.setUI(new MetalButtonUI());
        hullColorButton.setOpaque(true);
        hullColor = new Color(51, 204, 0);
        hullColorButton.setBackground(hullColor);
        hullColorButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color selectedColor = JColorChooser.showDialog(null,
                        "Choose group color", hullColor);
                if (selectedColor != null) {
                    hullColor = selectedColor;
                    hullColorButton.setBackground(hullColor);
                    Iterator displaysIt = displays.values().iterator();
                    while (displaysIt.hasNext()) {
                        Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                        cytoPrefuse.setHullColor(hullColor);
                    }
                }
            }
        });
        c.ipady = 10;
        c.gridx = 1;
        add(hullColorButton, c);

        JCheckBox checkShowSeparators = new JCheckBox("Show layer separators");
        checkShowSeparators.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                showSeparators = e.getStateChange() == ItemEvent.SELECTED;
                Iterator displaysIt = displays.values().iterator();
                while (displaysIt.hasNext()) {
                    Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                    cytoPrefuse.setShowSeparators(showSeparators);
                }
            }

        });
        checkShowSeparators.setSelected(true);
        checkShowSeparators.setAlignmentX(Component.LEFT_ALIGNMENT);

        c.fill = GridBagConstraints.HORIZONTAL;
        c.ipady = 0;
        c.gridx = 0;
        c.gridy = 21;
        c.gridwidth = 2;

        add(checkShowSeparators, c);

        JCheckBox checkHighQuality = new JCheckBox("High quality rendering");
        checkHighQuality.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                highQuality = e.getStateChange() == ItemEvent.SELECTED;
                firePropertyChange("quality", !highQuality, highQuality);
            }
        });
        checkHighQuality.setSelected(false);
        checkHighQuality.setAlignmentX(Component.LEFT_ALIGNMENT);

        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 22;
        c.gridwidth = 2;

        add(checkHighQuality, c);

        relayout = new JButton("Relayout");
        relayout.setEnabled(false);
        relayout.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (currentVis != null) {
                    ProgressMonitor progressMonitor = new ProgressMonitor(
                            Cytoscape.getDesktop().getNetworkViewManager()
                                    .getDesktopPane(), "Creating layout.", "",
                            0, 1000);
                    progressMonitor.setMillisToDecideToPopup(100);
                    progressMonitor.setMillisToPopup(400);
                    progressMonitor
                            .setNote("Estimated time left 1000 minutes.");

                    ProgressMonitor oldMonitor = currentVis.searchGridLayout
                            .getProgressMonitor();
                    if (oldMonitor != null) {
                        oldMonitor.close();
                    }
                    currentVis.searchGridLayout
                            .setProgressMonitor(progressMonitor);
                    currentVis.getVisualization().cancel("layout");
                    currentVis.getVisualization().run("layout");
                }
            }
        });
        c.gridy = 33;
        c.gridx = 0;
        c.gridwidth = 1;
        c.insets = new Insets(10, 0, 0, 0);
        c.fill = GridBagConstraints.NONE;
        add(relayout, c);

        c.insets = new Insets(20, 0, 0, 0);
        boolean inInnateDB = System.getProperty("javawslaunch")!=null;
        if (inInnateDB ) {
            // And add extra descriptors for InnateDB

            JPanel legendPanel = new JPanel();
            Insets noInsets = new Insets(0, 0, 0, 0);
            Insets topInset = new Insets(5,0,0,0);
            legendPanel.setLayout(new GridBagLayout());
            GridBagConstraints legendC = new GridBagConstraints();
            legendC.gridx = 0;
            legendC.gridy = 0;
            legendC.anchor = GridBagConstraints.LINE_START;
            legendC.weightx = 1.0;

            
            legendC.gridy = 6;
            legendC.insets = topInset;
            legendPanel.add(new JLabel("Supporting publications"),
                    legendC);
            JPanel interactLegend = new JPanel();
            interactLegend.setLayout(new GridBagLayout());
            legendC.insets = noInsets;
            legendC.gridy = 7;
            legendPanel.add(interactLegend, legendC);
            GridBagConstraints interactC = new GridBagConstraints();
            interactC.gridy = 0;

            Insets textInsets = new Insets(0, 5, 0, 20);
            interactC.gridx = 0;
            interactC.weightx = 0.0;
            interactC.insets = noInsets;
            interactLegend
                    .add(new LinePanel(StrokeLib.getStroke(1.0f, dashes)),
                            interactC);
            interactC.gridx = 1;
            interactC.weightx = 1.0;
            interactC.insets = textInsets;
            interactLegend.add(new JLabel("1 "), interactC);
            interactC.gridx = 2;
            interactC.weightx = 0.0;
            interactC.insets = noInsets;
            interactLegend.add(new LinePanel(StrokeLib.getStroke(1.0f)),
                    interactC);
            interactC.gridx = 3;
            interactC.weightx = 1.0;
            interactC.insets = textInsets;
            interactLegend.add(new JLabel("2 or 3 "), interactC);
            interactC.gridx = 4;
            interactC.weightx = 0.0;
            interactC.insets = noInsets;
            interactLegend.add(new LinePanel(StrokeLib.getStroke(2.0f)),
                    interactC);
            interactC.gridx = 5;
            interactC.weightx = 1.0;
            interactC.insets = textInsets;
            interactLegend.add(new JLabel("4 or more "), interactC);

            legendC.gridy = 8;
            legendC.insets = topInset;
            c.gridy = 39;
            c.anchor = GridBagConstraints.LINE_START;
            c.fill = GridBagConstraints.BOTH;
            add(legendPanel, c);
            c.insets = new Insets(5, 0, 0, 0);
        }

        c.gridx = 0;
        c.gridy = 40;
        c.weightx = 1.0;
        c.gridwidth = 2;
        c.fill = GridBagConstraints.HORIZONTAL;

        String expressionScale;
        if (!inInnateDB) {
            expressionScale = "c,-16.0,-15034815,c,0.0,-2680548,t,16.0,";
        }else{
            expressionScale = "c,-16.0,-15034815,c,0.0,-2680548,t,16.0,";
        }
        add(new JLabel("Expression color scale"), c);
        c.insets = new Insets(0, 0, 0, 0);
        c.gridy = 41;
        expressionColorLegend = new ColorLegend(
                ColorScaleBar
                        .loadFromString(expressionScale));
        add(expressionColorLegend, c);
        expressionColorLegend.addMouseListener(this);
        expressionColorLegend.setBackground(getBackground());

        c.gridy = 42;
        add(new JLabel("Comparison color scale"), c);
        c.gridy = 43;
        String comparisonScale;
        if (!inInnateDB){
            comparisonScale ="l,-2.5,-1679103,-526345,l,0.0,-526345,-10601319,t,2.5,";
        }else{
            comparisonScale =  "c,-16.0,-15034815,c,0.0,-2680548,t,16.0,";
        }
        comparisonColorLegend = new ColorLegend(
                ColorScaleBar
                        .loadFromString(comparisonScale));
        add(comparisonColorLegend, c);
        comparisonColorLegend.addMouseListener(this);
        comparisonColorLegend.setBackground(getBackground());

        JLabel sigCutoffLabel = new JLabel("Significance cut-off",
                JLabel.CENTER);

        sigCutoffVal = new JFormattedTextField(new DecimalFormat("#.###"));
        sigCutoffVal.setEditable(true);
        sigCutoffFilter = new JSlider(JSlider.HORIZONTAL, 0, 3000, 0);
        Hashtable sigLabels = new Hashtable();
        sigLabels.put(new Integer(0), new JLabel("1"));
        sigLabels.put(new Integer(1000), new JLabel("0.1"));
        sigLabels.put(new Integer(2000), new JLabel("0.01"));
        sigLabels.put(new Integer(3000), new JLabel("0.001"));

        sigCutoffFilter.setLabelTable(sigLabels);
        sigCutoffFilter.setPaintLabels(true);
        sigCutoffFilter.setPaintTicks(false);

        sigCutoffFilter.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                sigCutoff = (float) (1 / Math.pow(10.0,
                        source.getValue() / 1000.0));
                sigCutoffVal.setValue(new Float(sigCutoff));

                Iterator displaysIt = displays.values().iterator();
                while (displaysIt.hasNext()) {
                    Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                    cytoPrefuse.viewCoordinator.run("color");
                    cytoPrefuse.viewCoordinator.repaintAll();
                }
            }

        });
        sigCutoffFilter.setAlignmentX(Component.LEFT_ALIGNMENT);
        sigCutoffVal.addPropertyChangeListener("value", this);
        int sliderPos = (int) (1000 * Math.log10(1 / 0.05));
        sigCutoffFilter.setValue(sliderPos);

        c.gridx = 0;
        c.gridy = 44;
        add(sigCutoffLabel, c);
        c.gridx = 0;
        c.weightx = 0.5;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridy = 45;
        c.gridwidth = 1;

        add(sigCutoffFilter, c);
        c.gridx = 1;
        add(sigCutoffVal, c);

        JLabel foldChangeFilterLabel = new JLabel("Expression cutoff",
                JLabel.CENTER);

        foldChangeVal = new JFormattedTextField(new DecimalFormat("#.##"));
        foldChangeVal.setEditable(true);
        foldChangeFilter = new JSlider(JSlider.HORIZONTAL, 0, 40, 15);
        Hashtable foldChangeLabels = new Hashtable();
        foldChangeLabels.put(new Integer(0), new JLabel(PLUS_MINUS + "0"));
        foldChangeLabels.put(new Integer(10), new JLabel(PLUS_MINUS + "1"));
        foldChangeLabels.put(new Integer(20), new JLabel(PLUS_MINUS + "2"));
        foldChangeLabels.put(new Integer(30), new JLabel(PLUS_MINUS + "3"));
        foldChangeLabels.put(new Integer(40), new JLabel(PLUS_MINUS + "4"));

        foldChangeFilter.setLabelTable(foldChangeLabels);

        foldChangeFilter.setMajorTickSpacing(10);
        foldChangeFilter.setPaintLabels(true);
        foldChangeFilter.setPaintTicks(false);

        foldChangeFilter.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                foldChangeCutoff = source.getValue() / 10.0f;
                foldChangeVal.setValue(new Float(foldChangeCutoff));
                Iterator displaysIt = displays.values().iterator();
                while (displaysIt.hasNext()) {
                    Cerebral cytoPrefuse = (Cerebral) displaysIt.next();
                    cytoPrefuse.viewCoordinator.run("color");
                    cytoPrefuse.viewCoordinator.repaintAll();
                }
            }

        });
        foldChangeFilter.setAlignmentX(Component.LEFT_ALIGNMENT);
        foldChangeVal.addPropertyChangeListener("value", this);
        foldChangeVal.setValue(new Float(1.5));

        c.gridy = 46;
        c.gridx = 0;
        c.gridwidth = 2;
        add(foldChangeFilterLabel, c);
        c.weightx = 0.5;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridy = 47;
        c.gridwidth = 1;

        add(foldChangeFilter, c);
        c.gridx = 1;
        add(foldChangeVal, c);

        checkRatioData = new JCheckBox("Expression data are ratios");
        checkRatioData
                .setToolTipText("This box should be selected if your expression data represents ratios.  This is"
                        + " the case when a value of 1.0 represents no change and there are no expression values between -1.0 and 1.0");
        checkRatioData.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                isRatioData = e.getStateChange() == ItemEvent.SELECTED;
                firePropertyChange("ratioData", !isRatioData, isRatioData);
            }
        });
        checkRatioData.setSelected(false);
        checkRatioData.setAlignmentX(Component.LEFT_ALIGNMENT);

        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 48;
        c.gridwidth = 2;

        add(checkRatioData, c);

        JScrollPane scrollPane = new JScrollPane(groupsTable);
        scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);

        c.gridx = 0;
        c.gridwidth = 2;
        c.insets = new Insets(20, 0, 0, 0);
        c.gridy = 80;
        c.weighty = 0.95;
        c.fill = GridBagConstraints.BOTH;
        add(scrollPane, c);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));

        addGroup = new JButton("Add group");
        addGroup.setEnabled(false);
        addGroup.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (currentVis != null) {
                    currentVis.groupsManager.addGroup();
                    groupsTable.setRowSelectionInterval(groupsTable
                            .getRowCount() - 1, groupsTable.getRowCount() - 1);

                }
            }
        });

        deleteGroup = new JButton("Delete group");
        deleteGroup.setEnabled(false);
        deleteGroup.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                int selectedGroup = groupsTable.getSelectedRow();
                GroupsManager groupsManager = (GroupsManager) groupsTable
                        .getModel();
                groupsManager.removeGroup(selectedGroup);
                if (selectedGroup < groupsTable.getRowCount()) {
                    groupsTable.setRowSelectionInterval(selectedGroup,
                            selectedGroup);
                }
            }
        });

        buttonPanel.add(addGroup);
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(deleteGroup);
        buttonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);

        c.gridy = 81;
        c.weighty = 0.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.insets = new Insets(0, 0, 0, 0);
        add(buttonPanel, c);

    }

    public boolean getShowHulls() {
        return showHulls;
    }

    public boolean getShowSeparators() {
        return showSeparators;
    }

    public float getBeta() {
        return beta;
    }

    public float getDensity() {
        return labelDensity;
    }

    public Color getHullColor() {
        int alphaHullColor = ColorLib.rgba(hullColor.getRed(), hullColor
                .getGreen(), hullColor.getBlue(), 50);
        return ColorLib.getColor(alphaHullColor);
        // return hullColor;
    }

    public boolean isRatioData() {
        return isRatioData;
    }
    
    public void setIsRatioData(boolean newSetting){
    	checkRatioData.setSelected(newSetting);
    }

    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName() == CytoscapeDesktop.NETWORK_VIEW_FOCUSED) {
            currentVis = (Cerebral) displays.get(Cytoscape
                    .getCurrentNetworkView().getIdentifier());
            if (currentVis != null) {
                groupsTable.setModel(currentVis.groupsManager);
            }
            addGroup.setEnabled(currentVis != null);
            deleteGroup.setEnabled(groupsTable.getRowCount() > 0);
            relayout.setEnabled(currentVis != null);
            String colorScales = Cytoscape.getNetworkAttributes()
                    .getStringAttribute(
                            Cytoscape.getCurrentNetwork().getIdentifier(),
                            "cerebralActiveColorScales");
            if (colorScales != null) {
                String[] scales = colorScales.split("=");
                expressionColorLegend.setColorScale(ColorScaleBar
                        .loadFromString(scales[0]));
                comparisonColorLegend.setColorScale(ColorScaleBar
                        .loadFromString(scales[1]));

            }
        } else if (e.getPropertyName() == CytoscapeDesktop.NETWORK_VIEW_DESTROYED) {
            currentVis = null;
            groupsTable.setModel(new GroupsManager());
            addGroup.setEnabled(false);
            deleteGroup.setEnabled(false);
            relayout.setEnabled(false);
        } else if (e.getPropertyName() == "value") {
            try {
                if (e.getSource() == sigCutoffVal) {
                    JFormattedTextField slider = (JFormattedTextField) e
                            .getSource();
                    double sliderVal = Double.parseDouble(slider.getText());
                    int sliderPos = (int) (1000 * Math.log10(1 / sliderVal));
                    sigCutoffFilter.setValue(sliderPos);
                } else if (e.getSource() == foldChangeVal) {
                    JFormattedTextField slider = (JFormattedTextField) e
                            .getSource();
                    int sliderVal = (int) (Double.parseDouble(slider.getText()) * 10);
                    foldChangeFilter.setValue(sliderVal);

                }
            } catch (java.lang.NumberFormatException x) {
            }
        } else if (e.getPropertyName() == "newScale") {
            if (e.getNewValue() != null) {
                updatingLegend.setColorScale(ColorScaleBar
                        .loadFromString((String) e.getNewValue()));
                validate();
                CyAttributes globalAttributes = Cytoscape
                        .getNetworkAttributes();
                String saveString = csd.getSaveGallery();
                globalAttributes.setAttribute(Cytoscape.getCurrentNetwork()
                        .getIdentifier(), "cerebralColorScales", saveString);
                globalAttributes.setAttribute(Cytoscape.getCurrentNetwork()
                        .getIdentifier(), "cerebralActiveColorScales",
                        expressionColorLegend.getColorScaleBar().saveString()
                                + "="
                                + comparisonColorLegend.getColorScaleBar()
                                        .saveString());
                csd.dispose();

                if (currentVis != null) {
                    currentVis.viewCoordinator.run("color");
                    currentVis.viewCoordinator.repaintAll();
                }

            } else {
                validate();
                csd.dispose();
            }
        }
    }

    public void valueChanged(ListSelectionEvent e) {
        ListSelectionModel lsm = (ListSelectionModel) e.getSource();
        if (!lsm.isSelectionEmpty() && currentVis != null) {
            currentVis.groupsManager.setCurrentGroup(
                    lsm.getMaxSelectionIndex(), true);
        }
        deleteGroup.setEnabled(groupsTable.getRowCount() > 0
                && !lsm.isSelectionEmpty());
    }

    public float getGroupLabelSize() {
        return groupLabelSize;
    }

    public boolean getQuality() {
        String lcOSName = System.getProperty("os.name").toLowerCase();
        boolean MAC_OS_X = lcOSName.startsWith("mac os x");
        if (MAC_OS_X) {
            // Java2D on the mac charges no penalty for anti-aliasing. Always
            // return high quality
            return true;
        } else {
            return highQuality;
        }
    }

    public ColorRange getExpressionColor() {
        return expressionColorLegend.getColorScaleBar().getHead();
    }

    public ColorRange getComparisonColor() {
        return comparisonColorLegend.getColorScaleBar().getHead();
    }

    public void mouseClicked(MouseEvent e) {
        if (e.getSource() == expressionColorLegend
                || e.getSource() == comparisonColorLegend) {
            if (csd != null)
                csd.dispose();

            String userGallery = Cytoscape.getNetworkAttributes()
                    .getStringAttribute(
                            Cytoscape.getCurrentNetwork().getIdentifier(),
                            "cerebralColorScales");
            if (userGallery == null) {
                userGallery = "";
            }

            csd = new ColorScaleDialog(ColorScaleBar
                    .loadFromString(((ColorLegend) e.getSource())
                            .getColorScaleBar().saveString()), userGallery);

            updatingLegend = (ColorLegend) e.getSource();
            csd.addPropertyChangeListener("newScale", this);
            csd.showDialog();
        }

    }

    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public float getSigCutoff() {
        return sigCutoff;
    }

    public float getFoldChangeCutoff() {
        return foldChangeCutoff;
    }
}
