package cerebral.colors;

import java.awt.Graphics;

import prefuse.util.ColorLib;

public class LinearRange extends ColorRangeComponent {
	private static final long serialVersionUID = 1L;
	int colorMin, colorMax;
	
	public LinearRange (double min, ColorRange lowerRange, int colorMin, int colorMax){
		super(min,  lowerRange);
		this.colorMax = colorMax;
		this.colorMin = colorMin;
	}

	public int getColor(double val) {
		if (isInRange(val)){
			double frac = (val-min) / (higherRange.getMin() - min);
			return ColorLib.interp(colorMin, colorMax, frac);
		} else if (val < min) {
            // We're off the lefte side of the scale. Clip to min
            return colorMin;
        } else if (higherRange instanceof TerminateRange) {
            // off the right side
            return colorMax;
        } else {
			return higherRange.getColor(val);
		}
	}

	public void paint(Graphics g){
		int w = getWidth();
		
		for (int i=0; i < w; i++){
			double val = min + (i/(float)w) * (higherRange.getMin()-min);
			int color = getColor(val);
			g.setColor(ColorLib.getColor(color));
			g.drawLine(i,0, i, getHeight());
		}
		super.paint(g);
	}
}
