package cerebral.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.util.Iterator;

import javax.swing.SwingUtilities;

import cerebral.Cerebral;
import cerebral.ViewCoordinator;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.data.tuple.TupleSet;
import prefuse.util.PrefuseLib;
import prefuse.visual.VisualItem;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;

import prefuse.controls.ControlAdapter;

public class DividerMover extends ControlAdapter {

    protected Point2D down = new Point2D.Double();
    protected Point2D temp = new Point2D.Double();
    protected boolean dragged, resetItem;
    private Visualization m_vis;
    private TupleSet draggableItems;
    private double minY;
    private double maxY;
    private final String DELIM = "=";
    private ViewCoordinator viewCoordinator;
    /**
     * Creates a new drag control that issues repaint requests as an item
     * is dragged.
     */
    public DividerMover(ViewCoordinator viewCoordinator, Visualization vis, TupleSet draggableItems) {                
       m_vis = vis;
       this.draggableItems = draggableItems;
       this.viewCoordinator = viewCoordinator;    
    }
    
    
    /**
     * @see prefuse.controls.Control#itemEntered(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemEntered(VisualItem item, MouseEvent e) {
       if (draggableItems.containsTuple(item)){
           Display d = (Display)e.getSource();
           d.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
       }
    }
    
    /**
     * @see prefuse.controls.Control#itemExited(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemExited(VisualItem item, MouseEvent e) {
        if (draggableItems.containsTuple(item)){
            Display d = (Display)e.getSource();
            d.setCursor(Cursor.getDefaultCursor());
        }
    } //
    
    /**
     * @see prefuse.controls.Control#itemPressed(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemPressed(VisualItem item, MouseEvent e) {
        if (draggableItems.containsTuple(item)){
            if (!SwingUtilities.isLeftMouseButton(e)) return;
            dragged = false;
            Display d = (Display)e.getComponent();
            d.getAbsoluteCoordinate(e.getPoint(), down);
            Iterator it = draggableItems.tuples();
            while (it.hasNext()){
                VisualItem vi = (VisualItem) it.next();
                vi.setStartX(vi.getX());
                vi.setEndX(vi.getX());            
                vi.setStartY(vi.getY());
                vi.setEndY(vi.getY());                
            }
            int curLayer = item.getRow();
            
            minY = getPrevLayer(curLayer);                
            maxY = getNextLayer(curLayer);
                
            
        }
    }
    
    private double getNextLayer(int curLayer) {
        Iterator divIt = m_vis.getVisualGroup(Cerebral.LINE_DIVIDER).tuples();
        while (divIt.hasNext()){
            VisualItem divider = (VisualItem)divIt.next();
            if (divider.getRow() == curLayer +1){
                return divider.getY();
            }
        }

        return 1000000000.0;
    }
    private double getPrevLayer(int curLayer) {
        Iterator divIt = m_vis.getVisualGroup(Cerebral.LINE_DIVIDER).tuples();
        while (divIt.hasNext()){
            VisualItem divider = (VisualItem)divIt.next();
            if (divider.getRow() == curLayer -1){
                return divider.getY();
            }
        }
        return -1000000000.0;
    }


    /**
     * @see prefuse.controls.Control#itemReleased(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemReleased(VisualItem item, MouseEvent e) {
        if (draggableItems.containsTuple(item)){
            if (!SwingUtilities.isLeftMouseButton(e)) return;
            if ( dragged ) {
               save();
            }
        }
    }
 
    /**
     * @see prefuse.controls.Control#itemDragged(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemDragged(VisualItem item, MouseEvent e) {
        if (draggableItems.containsTuple(item)){
                 if (!SwingUtilities.isLeftMouseButton(e)) return;
            dragged = true;
            Display d = (Display)e.getComponent();
            d.getAbsoluteCoordinate(e.getPoint(), temp);
            double dy = temp.getY()-down.getY();
            if ((item.getY() + dy) > minY && item.getY()+dy < maxY){
                PrefuseLib.setY(item, null, item.getY() +dy);
                viewCoordinator.updateDividers();
                down.setLocation(temp);
            }
            
                 
        }
   }


    public void save() {
        String saveString = "";
        Iterator divIt = m_vis.getVisualGroup(Cerebral.LINE_DIVIDER).tuples();
        while (divIt.hasNext()){
            VisualItem divider = (VisualItem)divIt.next();
            saveString += divider.getRow();
            saveString += DELIM;
            saveString += divider.getY();
            saveString += DELIM;
        }
        CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
        
        globalAttributes.setAttribute(Cytoscape.getCurrentNetwork()
                .getIdentifier(), "cerebralDividers", saveString);
     
    }
    public void load(){
        CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
        
        String loadString = globalAttributes.getStringAttribute(Cytoscape.getCurrentNetwork()
                .getIdentifier(), "cerebralDividers");
        if (loadString != null){
                
            String[] vals = loadString.split(DELIM); 
            for (int i=0; i < vals.length; i+=2){
                String layerString = vals[i];
                int layer = Integer.parseInt(layerString);
                String posString = vals[i+1];
                Iterator divIt = m_vis.getVisualGroup(Cerebral.LINE_DIVIDER).tuples();
                while (divIt.hasNext()){
                    VisualItem divider = (VisualItem)divIt.next();
                    if (divider.getRow() == layer){
                        divider.setY(Double.parseDouble(posString));
                        break;
                    }
                }   
            }
            viewCoordinator.updateDividers();
        }else{
            viewCoordinator.positionDividers();
        }
        
    }

   
}
