package cerebral.controls;

import java.awt.Cursor;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.util.Iterator;

import javax.swing.SwingUtilities;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.controls.ControlAdapter;
import prefuse.data.tuple.TupleSet;
import prefuse.util.PrefuseLib;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import cerebral.Cerebral;
import cerebral.EdgeBundler;

public class GridDragger extends ControlAdapter {

    protected Point2D down = new Point2D.Double();

    protected Point2D temp = new Point2D.Double();

    protected boolean dragged, resetItem;

    private Visualization m_vis;

    private TupleSet draggableItems;

    private EdgeBundler edgeBundler;

    private Cerebral cytoPrefuse;

    /**
     * Creates a new drag control that issues repaint requests as an item is
     * dragged.
     * 
     * @param cytoPrefuse
     * @param searchGridLayout
     */
    public GridDragger(Visualization vis, TupleSet draggableItems,
            Cerebral cytoPrefuse, EdgeBundler edgeBundler) {

        m_vis = vis;
        this.draggableItems = draggableItems;
        this.edgeBundler = edgeBundler;
        this.cytoPrefuse = cytoPrefuse;
    }

    /**
     * @see prefuse.controls.Control#itemEntered(prefuse.visual.VisualItem,
     *      java.awt.event.MouseEvent)
     */
    public void itemEntered(VisualItem item, MouseEvent e) {
        if (item instanceof NodeItem) {
            Display d = (Display) e.getSource();
            d.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        }

    }

    /**
     * @see prefuse.controls.Control#itemExited(prefuse.visual.VisualItem,
     *      java.awt.event.MouseEvent)
     */
    public void itemExited(VisualItem item, MouseEvent e) {
        if (item instanceof NodeItem) {
            Display d = (Display) e.getSource();
            d.setCursor(Cursor.getDefaultCursor());
        }
    } //

    /**
     * @see prefuse.controls.Control#itemPressed(prefuse.visual.VisualItem,
     *      java.awt.event.MouseEvent)
     */
    public void itemPressed(VisualItem item, MouseEvent e) {
        if (item instanceof NodeItem) {
            if (!SwingUtilities.isLeftMouseButton(e))
                return;
            dragged = false;
            Display d = (Display) e.getComponent();
            d.getAbsoluteCoordinate(e.getPoint(), down);
            Iterator it = draggableItems.tuples();
            while (it.hasNext()) {
                VisualItem vi = (VisualItem) it.next();
                vi.setStartX(vi.getX());
              //  vi.setEndX(vi.getX());
                vi.setStartY(vi.getY());
              //  vi.setEndY(vi.getY());
            }
        }
    }

    /**
     * @see prefuse.controls.Control#itemReleased(prefuse.visual.VisualItem,
     *      java.awt.event.MouseEvent)
     */
    public void itemReleased(VisualItem item, MouseEvent e) {
        if (item instanceof NodeItem) {
            if (!SwingUtilities.isLeftMouseButton(e))
                return;
            if (dragged) {
                edgeBundler.recomputeLocations();
                cytoPrefuse.viewCoordinator.run("aggLayout");
                cytoPrefuse.labelLayout.setLabelPosition();
                cytoPrefuse.viewCoordinator.repaintAll();                
            }
        }
    }

    /**
     * @see prefuse.controls.Control#itemDragged(prefuse.visual.VisualItem,
     *      java.awt.event.MouseEvent)
     */
    public void itemDragged(VisualItem item, MouseEvent e) {
        if (item instanceof NodeItem) {
            if (!SwingUtilities.isLeftMouseButton(e))
                return;

            dragged = true;
            Display d = (Display) e.getComponent();
            d.getAbsoluteCoordinate(e.getPoint(), temp);
            double dx = temp.getX() - down.getX();
            double dy = temp.getY() - down.getY();

            TupleSet selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
            boolean allNodesAreSelected = allNodesAreSelected(selectedItems);
            if (!allNodesAreSelected && selectedItems.containsTuple(item)) {

                Iterator selectedIt = selectedItems.tuples();
                while (selectedIt.hasNext()) {
                    VisualItem vi = (VisualItem) selectedIt.next();
                    if (vi instanceof NodeItem) {
                        NodeItem nodeItem = (NodeItem) vi;
                        double x = nodeItem.getX();
                        double y = nodeItem.getY();

                        nodeItem.setStartX(x);
                        nodeItem.setStartY(y);
                        nodeItem.setX(x + dx);
                        nodeItem.setY(y + dy);
                        nodeItem.setEndX(x + dx);
                        nodeItem.setEndY(y + dy);
                        cytoPrefuse.setCytoscapePosition(nodeItem, true);
                    }
                }
            }else{
                PrefuseLib.setX(item, null, item.getX() + dx);
                PrefuseLib.setY(item, null, item.getY() + dy);
                cytoPrefuse.setCytoscapePosition((NodeItem) item, true);
            }

            cytoPrefuse.labelLayout.setLabelPosition();
            cytoPrefuse.viewCoordinator.repaintAll();

            down.setLocation(temp);
        }
    }

    private boolean allNodesAreSelected(TupleSet selectedItems) {
        Iterator nodesIt = m_vis.items(Cerebral.NODES);
        while (nodesIt.hasNext()) {
            NodeItem nodeItem = (NodeItem) nodesIt.next();
            if (nodeItem.isVisible()){
                if (!selectedItems.containsTuple(nodeItem)) {
                    return false;
                }
            }
        }

        return true;
    }
    
    public void keyReleased(KeyEvent e){
        System.out.println("KeyReleased");
    }

} // end of class Drag
