package cerebral.controls;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import cerebral.layout.SearchGridLayout;

import prefuse.Display;
import prefuse.data.tuple.TupleSet;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

import prefuse.controls.ControlAdapter;

public class NeighbourSelect extends ControlAdapter {

	 private List edges = new ArrayList();
	 private NodeItem focusNode;
	 private int currentNeighbour;
	 private TupleSet selectGroup;
	 private boolean canPan = false;
	 private NeighbourSorter neighbourSorter = new NeighbourSorter();
	 private Point2D panPoint = new Point2D.Float();
	 private boolean adjustRight = true;
	 public NeighbourSelect(TupleSet selectGroup) {
		 this.selectGroup = selectGroup;
	 }
	 public void itemEntered(VisualItem item, MouseEvent e) {
		if (item instanceof NodeItem) {			
			 focusNode = (NodeItem) item;
			 edges.clear();
			 if (selectGroup.containsTuple(item))
				 selectGroup.clear();
				
			 currentNeighbour = 0;			 
			 Iterator edgesIt = focusNode.edges();
			 while (edgesIt.hasNext() ){					  
				  EdgeItem edge = (EdgeItem) edgesIt.next();					 
				  edges.add(edge);
			 }
			 Collections.sort(edges, neighbourSorter);			 
		} 
	 }
	 public void itemExited(VisualItem item, MouseEvent e) {		 	
			canPan =false;
	 }
	public void itemWheelMoved(VisualItem item, MouseWheelEvent e) {
		 mouseWheelHandler(e);
	 }

	public void mouseWheelMoved(MouseWheelEvent e) {
	//	if (selectGroup.getTupleCount()>0) {
	//		mouseWheelHandler(e);
	//	}
	 }

	private void mouseWheelHandler (MouseWheelEvent e) {
		if (!edges.isEmpty()){	
			selectGroup.clear();
			
			currentNeighbour -= e.getWheelRotation();		
			currentNeighbour = currentNeighbour % edges.size();
			if (currentNeighbour < 0)
					currentNeighbour  += edges.size();
			EdgeItem currentEdge = (EdgeItem) edges.get(currentNeighbour);
			selectGroup.addTuple(currentEdge);
			canPan = true;
					
			NodeItem node = currentEdge.getAdjacentItem(focusNode);
			selectGroup.addTuple(node);
			Display d = (Display)e.getSource();
				
			d.getVisualization().run("color");		
	 	}
	 }
	
	
	public void keyTyped (KeyEvent e){
		  keyHandler(e);	  
	}
	private void keyHandler(KeyEvent e) {
	   if (e.getKeyChar() == ' '){
		  panToNeighbour((Display) e.getSource());
	   }
	}
    public void itemKeyTyped(VisualItem item, KeyEvent e) {
		keyHandler(e);
    } 
	   private void panToNeighbour (Display d)  {
			  if (canPan) {
				  Iterator neighboursIt = selectGroup.tuples();
				  while (neighboursIt.hasNext()) {
					  VisualItem item = (VisualItem) neighboursIt.next();
					  if (item instanceof NodeItem) {
						  //we don't want to pan under the mouse or we won't trigger 
						  //mouse events.  Pan just to the right or left of the mouse
						  //pointer.
						  int oscillation;
						  if (adjustRight)
							  oscillation = (int) (SearchGridLayout.SCALE/3);
						  else
							  oscillation = (int) (-SearchGridLayout.SCALE/3);
						  adjustRight = ! adjustRight;
						  panPoint.setLocation(item.getX() + oscillation, item.getY());
						  d.animatePanToAbs(panPoint, 1000);
						
						  break;
					  }
				  }
			  }
	  }
	
	private class NeighbourSorter implements Comparator {
		
		public int compare(Object arg0, Object arg1) {
			if (arg1 == null)
				return -1;
			if (arg0 == null)
				return 1;
			EdgeItem edge = (EdgeItem)arg0;
			NodeItem node = edge.getAdjacentItem(focusNode);
			double deltaX = focusNode.getX() - node.getX();
			double deltaY = focusNode.getY() - node.getY();
			double theta1 = Math.atan2(deltaY, deltaX);	
			
			edge = (EdgeItem)arg1;
			NodeItem otherNode = edge.getAdjacentItem(focusNode);
			deltaX = focusNode.getX() - otherNode.getX();
			deltaY = focusNode.getY() - otherNode.getY();
			double theta2 = Math.atan2(deltaY, deltaX);	
			
			if (theta1 < theta2) {
				return -1;
			}else if (theta1 == theta2) {
				return 0;
			}else 
				return 1;
		}
	 }

}
