package cerebral.parallelCoordinates;

import java.awt.Font;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;

import cerebral.parallelCoordinates.render.DataRenderer;
import cerebral.parallelCoordinates.render.IntervalLabelRenderer;
import cerebral.parallelCoordinates.render.ScaleLineRenderer;

import prefuse.action.layout.Layout;
import prefuse.data.tuple.TupleSet;
import prefuse.render.Renderer;
import prefuse.util.FontLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.VisualItem;

public class PCLayout extends Layout {
	private DataRenderer pcRenderer;

	private IntervalRenderer intervalRenderer;

	private IntervalLabelRenderer intervalLabelRendererTop;

	private IntervalLabelRenderer intervalLabelRendererBottom;

	private ScaleLineRenderer scaleLineRenderer;

	public static final int MARGIN = 10;

	private double xScale = 0.0;

	private YDisplay yDisplay;

	private boolean userControlled;

	private String commonPrefix;

	public PCLayout(String group, DataRenderer pcRenderer,
			IntervalRenderer intervalRenderer, YDisplay yDisplay,
			IntervalLabelRenderer intervalLabelRendererTop,
			IntervalLabelRenderer intervalLabelRendererBottom,
			ScaleLineRenderer axisRenderer, String commonPrefix) {
		super(group);
		this.pcRenderer = pcRenderer;
		this.intervalRenderer = intervalRenderer;
		this.yDisplay = yDisplay;
		this.intervalLabelRendererTop = intervalLabelRendererTop;
		this.intervalLabelRendererBottom = intervalLabelRendererBottom;
		this.scaleLineRenderer = axisRenderer;
		this.commonPrefix = commonPrefix;
		userControlled = false;
	}

	public void setXScale(double xScale) {
		this.xScale = xScale;
	}

	public void setUserControlled(boolean b) {
		userControlled = b;
	}

	public void run(double frac) {

		int fontSize = 14;
		Font font = FontLib.getFont("Verdana", fontSize);
		FontRenderContext frc = Renderer.DEFAULT_GRAPHICS
				.getFontRenderContext();

		Iterator labelsIt = m_vis.getGroup(
				ParallelCoordinates.COORDINATE_LABELS).tuples();

		VisualItem item = (VisualItem) labelsIt.next();
		Renderer r = item.getRenderer();
		r.setBounds(item);
		double leftMargin = item.getBounds().getWidth();

		// Create an extra margin for prefix label
		double prefixMargin = 0;
		if (commonPrefix.length() > 0) {
			TextLayout textLayout = new TextLayout(commonPrefix, font, frc);
			prefixMargin = textLayout.getBounds().getWidth();
		}

		double labelMaxHeight = 0;

		labelsIt = m_vis.getGroup(ParallelCoordinates.COORDINATE_LABELS)
				.tuples();
		while (labelsIt.hasNext()) {
			item = (VisualItem) labelsIt.next();
			r.setBounds(item);
			if (item.getBounds().getHeight() > labelMaxHeight) {
				labelMaxHeight = item.getBounds().getHeight();
			}
		}

		TupleSet coordinates = m_vis.getGroup(ParallelCoordinates.COORDINATES);

		double width = m_vis.getDisplay(0).getWidth() - leftMargin
				- prefixMargin;
		double height = m_vis.getDisplay(0).getHeight() - MARGIN / 2
				- labelMaxHeight;
		double unit = (width - 4 * MARGIN) / (coordinates.getTupleCount() - 1);

		// sort based on current order
		VisualItem[] sortedCoords = new VisualItem[coordinates.getTupleCount()];
		{
			Iterator coordinatesIt = coordinates.tuples();
			int j = 0;
			while (coordinatesIt.hasNext()) {
				sortedCoords[j++] = (VisualItem) coordinatesIt.next();
			}
			Arrays.sort(sortedCoords, new XOderCompare());
		}
		for (int i = 0; i < sortedCoords.length; i++) {
			item = sortedCoords[i];
			if (!userControlled || xScale == 0.0) {
				item.setX(unit * i + MARGIN + leftMargin + prefixMargin);
			} else {
				item.setX(item.getX() * xScale);
			}
			item.setY(0);
			item.setSize(height);
		}

		Iterator filtersIt = m_vis.getGroup(ParallelCoordinates.FILTERS)
				.tuples();
		while (filtersIt.hasNext()) {
			VisualItem filter = (VisualItem) filtersIt.next();
			if (xScale != 0.0) {
				filter.setX(filter.getX() * xScale);
			}
		}
		pcRenderer.validate();
		pcRenderer.setBounds(leftMargin, 0, width, height);

		yDisplay.setDisplayHeight(height);
		intervalRenderer.setBounds(0, height);
		intervalLabelRendererTop.setBounds(0, height);
		intervalLabelRendererBottom.setBounds(0, height);
		scaleLineRenderer.setBounds(0, height);

		labelsIt = m_vis.getGroup(ParallelCoordinates.COORDINATE_LABELS)
				.tuples();
		while (labelsIt.hasNext()) {
			DecoratorItem dItem = (DecoratorItem) labelsIt.next();
			dItem.setX(dItem.getDecoratedItem().getX());
			// dItem.setY(m_vis.getDisplay(0).getHeight() -
			// textBounds.getHeight()
			// / 2 - MARGIN / 2);
			dItem.setY(height);
		}
	}

	class XOderCompare implements Comparator {

		public int compare(Object arg0, Object arg1) {
			VisualItem a = (VisualItem) arg0;
			VisualItem b = (VisualItem) arg1;

			if (a.getX() > b.getX()) {
				return 1;
			} else {
				return -1;
			}
		}
	};

}
