package cerebral.parallelCoordinates;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import cerebral.ControlPanel;
import cerebral.parallelCoordinates.controls.TooltipSliderUI;
import cytoscape.Cytoscape;
import cytoscape.view.CytoscapeDesktop;

public class PCPanel extends JPanel implements PropertyChangeListener {
    private static final long serialVersionUID = 1L;

    private Map PCs = new HashMap();

    private JSplitPane splitPane;

    private JScrollPane clusterPane;

    private ParallelCoordinates currentPC;

    private Component border;

    private JSlider kSlider;

    private static PCPanel panel = new PCPanel();

    public static PCPanel instance() {
        return panel;
    }

    private PCPanel() {
        Cytoscape.getDesktop().getSwingPropertyChangeSupport()
                .addPropertyChangeListener(this);
        Cytoscape.getSwingPropertyChangeSupport().addPropertyChangeListener(
                this);

        ControlPanel.instance().addPropertyChangeListener(this);

        clusterPane = new JScrollPane();
        clusterPane.getViewport().addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                if (currentPC != null) {
                    currentPC.getClusterDisplay().setSize(
                            currentPC.getPreferredClusterSize(clusterPane
                                    .getViewport().getHeight()),
                            clusterPane.getViewport().getHeight());
                }
            }

        });
        clusterPane
                .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        clusterPane
                .setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        JPanel clusterPanel = new JPanel();
        clusterPanel.setLayout(new BoxLayout(clusterPanel, BoxLayout.Y_AXIS));
        clusterPanel.add(clusterPane);
        kSlider = new JSlider(JSlider.HORIZONTAL, 0, 50, 10);
        TooltipSliderUI tts = new TooltipSliderUI(kSlider);
        kSlider.setUI(tts);
        kSlider.setMajorTickSpacing(10);
        kSlider.setMinorTickSpacing(1);
        kSlider.setPaintLabels(true);
        kSlider.setPaintTicks(true);
        kSlider.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                if (!source.getValueIsAdjusting()) {
                    int k = source.getValue();
                    if (currentPC != null) {
                        currentPC.setK(k);
                    }

                }
            }
        });
        clusterPanel.add(kSlider);
        JPanel pcControlPanel = new JPanel();
        pcControlPanel
                .setLayout(new BoxLayout(pcControlPanel, BoxLayout.X_AXIS));

        JButton selectAttributes = new JButton();
        selectAttributes.setIcon(new ImageIcon(getClass().getResource(
                "icons/select-attributes.png")));
        selectAttributes.setToolTipText("Select expression value attributes");
        selectAttributes.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                if (currentPC != null) {
                    currentPC.selectAttributes();
                }
            }
        });

        pcControlPanel.add(selectAttributes);
        pcControlPanel.add(Box.createHorizontalGlue());

        JButton zoomToFit = new JButton();
        zoomToFit.setIcon(new ImageIcon(getClass().getResource(
                "icons/zoom-selected.png")));
        zoomToFit
                .setToolTipText("Zoom parallel coordinates to selected region");
        zoomToFit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                if (currentPC != null) {
                    currentPC.zoomToFit();
                }
            }
        });
        pcControlPanel.add(zoomToFit);

        JButton resetZoom = new JButton();
        resetZoom.setIcon(new ImageIcon(getClass().getResource(
                "icons/zoom-all.png")));

        resetZoom
                .setToolTipText("Zoom out to display all parallel coordinates data lines");
        resetZoom.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (currentPC != null) {
                    currentPC.zoomAll();
                }
            }

        });
        pcControlPanel.add(resetZoom);

        JButton alignAxes = new JButton ("");
        alignAxes.setIcon(new ImageIcon(getClass().getResource(
        "icons/align2.png")));
        alignAxes.setToolTipText("Set even spacing between axes");
        pcControlPanel.add(alignAxes);
        alignAxes.addActionListener((new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                if (currentPC != null) {
                	currentPC.alignAxes();
                }
            }        	
        }));
        
        pcControlPanel.add(Box.createHorizontalGlue());

        JButton ignore = new JButton();
        ignore.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (currentPC != null) {
                    currentPC.filterSelected();
                }
            }
        });
        ignore
                .setIcon(new ImageIcon(getClass().getResource(
                        "icons/filter.png")));
        ignore.setToolTipText("Filter selected data lines");

        pcControlPanel.add(ignore);

        JButton restore = new JButton();
        restore.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (currentPC != null) {
                    currentPC.unfilterAll();
                }
            }

        });
        restore.setIcon(new ImageIcon(getClass().getResource(
                "icons/filter-off.png")));
        restore.setToolTipText("Restore all filtered data lines");

        pcControlPanel.add(Box.createHorizontalStrut(10));
        pcControlPanel.add(restore);

        clusterPanel.add(pcControlPanel);

        JPanel pcPanel = new JPanel();
        pcPanel.setLayout(new BoxLayout(pcPanel, BoxLayout.Y_AXIS));
        splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pcPanel,
                clusterPanel);
        splitPane.setResizeWeight(1.0);
        splitPane.setDividerLocation(0.9);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        add(splitPane);
        border = Box.createRigidArea(new Dimension(0, 5));
    }

    public void addPC(ParallelCoordinates PC, String identifier) {
        PCs.put(identifier, PC);
    }

    public void removePC(String identifier) {
        PCs.remove(identifier);
    }

    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName() == CytoscapeDesktop.NETWORK_VIEW_FOCUSED) {
            currentPC = (ParallelCoordinates) PCs.get(Cytoscape
                    .getCurrentNetwork().getIdentifier());
            if (currentPC != null) {
                JPanel pcPanel = (JPanel) splitPane.getLeftComponent();
                pcPanel.removeAll();
                pcPanel.add(border);
                pcPanel.add(currentPC.getDisplay());
                // splitPane.setLeftComponent(currentPC.getDisplay());
                clusterPane.setViewportView(currentPC.getClusterDisplay());

            }
        } else if (e.getPropertyName() == Cytoscape.NETWORK_DESTROYED) {
            currentPC = null;
            ParallelCoordinates pc = getPC((String) e.getNewValue());
            if (pc != null) {
                removePC((String) e.getNewValue());
                pc.disconnect();
            }
            clusterPane.setViewportView(null);
            // splitPane.setLeftComponent(null);
            JPanel pcPanel = (JPanel) splitPane.getLeftComponent();
            pcPanel.removeAll();
        } else if (e.getPropertyName() == Cytoscape.NETWORK_CREATED) {
            addPC(new ParallelCoordinates((String) e.getNewValue(), splitPane
                    .getLeftComponent()), Cytoscape.getCurrentNetwork()
                    .getIdentifier());
        } else if (e.getPropertyName().compareTo("ratioData") == 0) {
            if (currentPC != null) {
                currentPC.setK(kSlider.getValue());
            }

        }
    }

    public ParallelCoordinates getPC(String networkID) {
        return (ParallelCoordinates) PCs.get(networkID);
    }
}
