package cerebral.parallelCoordinates;

import giny.view.NodeView;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.tuple.DefaultTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.PolygonRenderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.PrefuseLib;
import prefuse.util.StrokeLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.VisualGraph;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;
import cerebral.Cerebral;
import cerebral.ControlPanel;
import cerebral.NetworksManager;
import cerebral.parallelCoordinates.cluster.ClusterGlyphColor;
import cerebral.parallelCoordinates.cluster.ClusterSelector;
import cerebral.parallelCoordinates.cluster.KMeansCluster;
import cerebral.parallelCoordinates.controls.CoordinateDragger;
import cerebral.parallelCoordinates.controls.DataToolTip;
import cerebral.parallelCoordinates.controls.LineSelect;
import cerebral.parallelCoordinates.controls.RubberBandSelect;
import cerebral.parallelCoordinates.controls.ScaleControl;
import cerebral.parallelCoordinates.controls.VerticalPanControl;
import cerebral.parallelCoordinates.render.ClusterGlyphRenderer;
import cerebral.parallelCoordinates.render.CoordinateRenderer;
import cerebral.parallelCoordinates.render.DataRenderer;
import cerebral.parallelCoordinates.render.IntervalLabelRenderer;
import cerebral.parallelCoordinates.render.RotatedTextRenderer;
import cerebral.parallelCoordinates.render.ScaleLineRenderer;
import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;
import cytoscape.data.CyAttributesUtils;
import cytoscape.data.SelectEvent;
import cytoscape.data.SelectEventListener;
import cytoscape.view.CyNetworkView;
import cytoscape.visual.GlobalAppearance;
import cytoscape.visual.GlobalAppearanceCalculator;
import cytoscape.visual.VisualStyle;

public class ParallelCoordinates extends JPanel implements SelectEventListener,
		PropertyChangeListener, ChangeListener {

	private static final long serialVersionUID = 1L;

	public static final String IGNORE = "ignore";

	public static final String CLUSTER_GLYPH_COLOR = "ClusterGlyphColor";

	public static final String CLUSTERS = "clusters";

	public static final String COLOR = "color";

	public static final String DATA = "data";

	public static final String INTERVAL = "interval";

	public static final String COORDINATE_LABELS = "coordinateLabels";

	public static final String COORDINATES = "coordinates";

	public static final String FILTERS = "filters";

	public static final String FILTER_TOP_LABELS = "filterTopLabels";

	public static final String FILTER_BOTTOM_LABELS = "filterBottomLabels";

	public static final String DECORATOR = "decorator";

	public static final String LABEL = "label";

	public static final String RUBBER_BAND = "rubberBand";

	public static final String SELECTED = "selected";

	public static final String FILTER = "filter";

	private static final String INTERACTIVITY = "interactivity";

	public static final String VALUES = "values";

	public static final String NORM_VALUES = "normValues";

	public static final String TOP = "top";

	public static final String BOTTOM = "bottom";

	public static final String COORDINATE = "coordinate";

	public static final String TEMP_FILTER = "tempFilter";

	public static final String TEXT_EDIT = "textEdit";

	private Table filterTable;

	private double minData, maxData;

	private CoordinateDragger coordinateDragger;

	private RubberBandSelect rubberBandSelect;

	private DataToolTip dataToolTip;

	private LineSelect lineSelect;

	private ScaleControl scaleControl;

	private VerticalPanControl verticalPanControl;

	private ClusterSelector clusterSelector;

	private Display d;

	private Visualization vis;

	private Display clusterD;

	private Visualization clusterVis;

	private PCLayout pcLayout;

	private int lastX;

	private DataRenderer dataRenderer;

	private Map idToVisData;

	private IntervalRenderer intervalRenderer;

	private DataFilterer dataFilterer;

	private KMeansCluster kMeans;

	private Table clusterTable;

	private CyNetwork cyNetwork;

	private VisualItem rubberBand;

	private ClusterGlyphRenderer clusterGlyphRenderer;

	private AttributeChooser attributeChooser = new AttributeChooser();

	private YDisplay yDisplay = new YDisplay();

	private Component border;

	private final String DELIM = "=";

	private VisualItem primarySelection;
	private VisualItem secondarySelection;

	private static final Schema LABEL_SCHEMA = PrefuseLib.getVisualItemSchema();
	static {
		LABEL_SCHEMA.setDefault(VisualItem.INTERACTIVE, true);
		LABEL_SCHEMA.setDefault(VisualItem.TEXTCOLOR, ColorLib.gray(0));
		LABEL_SCHEMA.setDefault(VisualItem.FILLCOLOR, ColorLib.rgba(255, 255,
				255, 150));
		LABEL_SCHEMA
				.setDefault(VisualItem.FONT, FontLib.getFont("Verdana", 14));
	}

	public ParallelCoordinates(String networkName, Component border) {
		vis = new Visualization();
		d = new Display(vis);
		this.border = border;
		clusterVis = new Visualization();
		clusterD = new Display(clusterVis);

		cyNetwork = Cytoscape.getNetwork(networkName);
		lastX = 0;

		d.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				int x = d.getWidth();
				double dx = x / (double) lastX;
				if (pcLayout != null) {
					pcLayout.setXScale(dx);
					vis.run("layout");
				}
				lastX = d.getWidth();

			}
		});

		loadSavedAxes();
		cyNetwork.addSelectEventListener(this);
		Cytoscape.getVisualMappingManager().addChangeListener(this);

	}

	public void disconnect() {
		cyNetwork.removeSelectEventListener(this);
		ControlPanel.instance().removePropertyChangeListener(this);
		Cytoscape.getVisualMappingManager().removeChangeListener(this);

	}

	public void loadData(List coordinateNames, List data, List dataLabels) {

		removeAll();
		vis.reset();

		dataFilterer = new DataFilterer(d);

		Table dataTable = new Table();
		dataTable.addColumn(LABEL, String.class);
		dataTable.addColumn("cluster", int.class);
		dataTable.addColumn(IGNORE, boolean.class);
		dataTable.addColumn(VALUES, List.class);
		dataTable.addColumn(NORM_VALUES, List.class);
		for (int i = 0; i < data.size(); i++) {
			int id = dataTable.addRow();
			dataTable.set(id, LABEL, dataLabels.get(i));
			dataTable.setInt(id, "cluster", 0);
			dataTable.setBoolean(id, IGNORE, false);
			dataTable.set(id, VALUES, data.get(i));
			List rawValues = (List) data.get(i);
			double length = 0.0;
			for (int j = 0; j < rawValues.size(); j++) {
				length += ((Double) rawValues.get(j)).doubleValue()
						* ((Double) rawValues.get(j)).doubleValue();
			}
			length = Math.sqrt(length);
			if (Math.abs(length) < 0.00001) {
				length = 1.0;
			}
			List normValues = new ArrayList();
			for (int j = 0; j < rawValues.size(); j++) {
				normValues.add(new Double(((Double) rawValues.get(j))
						.doubleValue()
						/ length));
			}
			dataTable.set(id, NORM_VALUES, normValues);
		}

		vis.add(DATA, dataTable);

		idToVisData = new HashMap();
		Iterator dataIt = vis.getVisualGroup(DATA).tuples();
		while (dataIt.hasNext()) {
			VisualItem item = (VisualItem) dataIt.next();
			idToVisData.put(item.getString(LABEL), item);
			item.setStrokeColor(ColorLib.gray(0));
		}

		minData = 100000000.0;
		maxData = -100000000.0;
		Iterator dataItems = vis.items(DATA);
		while (dataItems.hasNext()) {
			VisualItem dataItem = (VisualItem) dataItems.next();
			List values = (List) dataItem.get(VALUES);
			Iterator valuesIt = values.iterator();
			while (valuesIt.hasNext()) {
				Double val = (Double) valuesIt.next();
				if (val.doubleValue() < minData) {
					minData = val.doubleValue();
				}
				if (val.doubleValue() > maxData) {
					maxData = val.doubleValue();
				}
			}
		}

		double dataRange = (Math.abs(maxData) > Math.abs(minData)) ? Math
				.abs(maxData) : Math.abs(minData);
		dataRange *= 2;

		// Let's see if our coordinate axes have a common prefix to the names
		String commonPrefix = "";
		/*
		 * I think the common prefix looked goofy. if (coordinateNames.size() >
		 * 0) { int prefixLength = 1; String firstName = (String)
		 * coordinateNames.get(0); boolean continuePrefixSearch = true; do { if
		 * (firstName.length() >= prefixLength) { char testPrefix =
		 * firstName.charAt(prefixLength - 1); for (int i = 1; i <
		 * coordinateNames.size(); i++) { String otherName = (String)
		 * coordinateNames.get(i); if (otherName.length() >= prefixLength) { if
		 * (otherName.charAt(prefixLength - 1) != testPrefix) {
		 * continuePrefixSearch = false; break; } } } if (continuePrefixSearch) {
		 * prefixLength++; } } } while (continuePrefixSearch); commonPrefix =
		 * firstName.substring(0, prefixLength-1); } System.out.println("Common
		 * axis prefix is " + commonPrefix);
		 */
		Table coordinates = new Table();
		coordinates.addColumn(LABEL, String.class);
		coordinates.addColumn(DECORATOR, VisualItem.class);
		coordinates.addColumn(TEMP_FILTER, int.class);
		for (int i = 0; i < coordinateNames.size(); i++) {
			int r = coordinates.addRow();
			String coordinateName = (String) coordinateNames.get(i);
			coordinates.setString(r, LABEL, coordinateName.substring(
					commonPrefix.length(), coordinateName.length()));
		}
		vis.add(COORDINATES, coordinates);
		vis.setInteractive(COORDINATES, null, true);
		Iterator coordinateIt = vis.getVisualGroup(COORDINATES).tuples();
		while (coordinateIt.hasNext()) {
			VisualItem item = (VisualItem) coordinateIt.next();
			item.setStroke(StrokeLib.getStroke(2));
			item.setFillColor(ColorLib.gray(0));
		}
		vis.addDecorators(COORDINATE_LABELS, COORDINATES);
		Iterator labelsIt = vis.getVisualGroup(COORDINATE_LABELS).tuples();
		while (labelsIt.hasNext()) {
			DecoratorItem item = (DecoratorItem) labelsIt.next();
			item.setTextColor(ColorLib.gray(0));
			item.setInteractive(true);
			item.setFont(FontLib.getFont("verdana", 14));
			VisualItem coordinate = item.getDecoratedItem();
			coordinate.set(DECORATOR, item);
		}

		filterTable = new Table();
		filterTable.addColumn(TOP, double.class);
		filterTable.addColumn(BOTTOM, double.class);
		filterTable.addColumn(COORDINATE, VisualItem.class);
		filterTable.addColumn(TEXT_EDIT, String.class);
		vis.addTable(FILTERS, filterTable);
		vis.addDecorators(FILTER_TOP_LABELS, FILTERS, LABEL_SCHEMA);
		vis.addDecorators(FILTER_BOTTOM_LABELS, FILTERS, LABEL_SCHEMA);

		Table rubberBandTable = new Table();
		rubberBandTable.addColumn(VisualItem.POLYGON, float[].class);
		int r = rubberBandTable.addRow();
		rubberBandTable.set(r, VisualItem.POLYGON, new float[8]);
		vis.add(RUBBER_BAND, rubberBandTable);
		rubberBand = (VisualItem) vis.getVisualGroup(RUBBER_BAND).tuples()
				.next();
		rubberBand.setStrokeColor(ColorLib.color(ColorLib.getColor(255, 0, 0)));
		rubberBand.setVisible(false);

		// add a single item to render as the scale axis
		Table scaleTable = new Table();
		scaleTable.addRow();
		vis.add("scale", scaleTable);
		VisualItem scaleItem = (VisualItem) vis.getVisualGroup("scale")
				.tuples().next();
		scaleItem.setInteractive(false);
		float[] dashes = { 1.0f, 2.0f };
		scaleItem.setStroke(new BasicStroke(2.0f, BasicStroke.CAP_SQUARE,
				BasicStroke.JOIN_MITER, 10.0f, dashes, 0f));

		TupleSet selectedItems;
		if (vis.getFocusGroup(SELECTED) == null) {
			selectedItems = new DefaultTupleSet();
			vis.addFocusGroup(SELECTED, selectedItems);
		} else {
			selectedItems = vis.getFocusGroup(SELECTED);
		}

		d.setHighQuality(ControlPanel.instance().getQuality());
		ControlPanel.instance().addPropertyChangeListener(this);

		DefaultRendererFactory drf = new DefaultRendererFactory();
		drf.add(new InGroupPredicate(COORDINATES), new CoordinateRenderer());
		drf.add(new InGroupPredicate(COORDINATE_LABELS),
				new RotatedTextRenderer(-30));
		// drf.add(new InGroupPredicate(COORDINATE_LABELS), new
		// LabelRenderer());

		int dimensions = 0;
		if (data.size() > 0) {
			List values = (List) data.get(0);
			dimensions = values.size();
		}
		dataRenderer = new DataRenderer(d, yDisplay, dimensions);
		drf.add(new InGroupPredicate(DATA), dataRenderer);
		drf.add(new InGroupPredicate(RUBBER_BAND), new PolygonRenderer(
				Constants.POLY_TYPE_LINE));
		ScaleLineRenderer scaleLineRenderer = new ScaleLineRenderer(yDisplay,
				commonPrefix);
		drf.add(new InGroupPredicate("scale"), scaleLineRenderer);
		intervalRenderer = new IntervalRenderer(yDisplay);
		drf.add(new InGroupPredicate(FILTERS), intervalRenderer);
		IntervalLabelRenderer intervalLabelRendererTop = new IntervalLabelRenderer(
				yDisplay, TOP);
		drf.add(new InGroupPredicate(FILTER_TOP_LABELS),
				intervalLabelRendererTop);
		IntervalLabelRenderer intervalLabelRendererBottom = new IntervalLabelRenderer(
				yDisplay, BOTTOM);
		drf.add(new InGroupPredicate(FILTER_BOTTOM_LABELS),
				intervalLabelRendererBottom);

		vis.setRendererFactory(drf);

		kMeans = new KMeansCluster(vis, dimensions);

		kMeans.cluster(10);

		clusterGlyphRenderer = new ClusterGlyphRenderer(d, clusterD,
				dimensions, dataRange);

		pcLayout = new PCLayout(COORDINATES, dataRenderer, intervalRenderer,
				yDisplay, intervalLabelRendererTop,
				intervalLabelRendererBottom, scaleLineRenderer, commonPrefix);
		ActionList layout = new ActionList();
		layout.add(pcLayout);
		layout.add(new RepaintAction());
		vis.putAction("layout", layout);
		vis.run("layout");
		vis.putAction("zoomToFit", new ZoomPCToFitAction(yDisplay, minData,
				maxData));
		vis.runAfter("layout", "zoomToFit");

		vis.putAction("filterLayout", new FilterLayout());
		ActionList colorList = new ActionList();
		MyColorAction color = new MyColorAction(DATA, vis);
		colorList.add(color);
		MyStrokeAction stroke = new MyStrokeAction(DATA);
		colorList.add(stroke);
		colorList.add(new RepaintAction());
		vis.putAction(COLOR, colorList);
		vis.run(COLOR);

		ActionList interactivity = new ActionList();
		interactivity.add(new InteractiveFilter(DATA, selectedItems));
		vis.putAction(INTERACTIVITY, interactivity);
		vis.run(INTERACTIVITY);

		d.removeControlListener(coordinateDragger);
		d.removeControlListener(rubberBandSelect);
		d.removeControlListener(dataToolTip);
		d.removeControlListener(lineSelect);
		d.removeControlListener(scaleControl);
		d.removeControlListener(verticalPanControl);

		coordinateDragger = new CoordinateDragger(this, vis, dimensions,
				dataRenderer, clusterGlyphRenderer, commonPrefix);
		rubberBandSelect = new RubberBandSelect(this, yDisplay, filterTable,
				rubberBand);
		dataToolTip = new DataToolTip(this);
		scaleControl = new ScaleControl(yDisplay);
		verticalPanControl = new VerticalPanControl(yDisplay);
		lineSelect = new LineSelect(this);

		d.addControlListener(coordinateDragger);
		d.addControlListener(rubberBandSelect);
		d.addControlListener(dataToolTip);
		d.addControlListener(scaleControl);
		d.addControlListener(verticalPanControl);
		d.addControlListener(lineSelect);

		d.setItemSorter(new MyItemSorter());

		clusterVis.reset();
		clusterTable = new Table();
		clusterTable.addColumn("dataPoints", List.class);
		clusterTable.addColumn("count", int.class);
		clusterVis.addTable(CLUSTERS, clusterTable);

		DefaultRendererFactory clusterDrf = new DefaultRendererFactory();
		clusterDrf.add(new InGroupPredicate(CLUSTERS), clusterGlyphRenderer);
		clusterVis.setRendererFactory(clusterDrf);
		clusterD.removeControlListener(clusterSelector);
		clusterSelector = new ClusterSelector(kMeans, vis, this);
		clusterD.addControlListener(clusterSelector);
		clusterD.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				clusterLayout(clusterD);
			}
		});
		clusterLayout(clusterD);
		ActionList clusterColorList = new ActionList();
		ClusterGlyphColor clusterGlyphColor = new ClusterGlyphColor(CLUSTERS,
				kMeans, vis);
		clusterColorList.add(clusterGlyphColor);
		clusterColorList.add(new RepaintAction());
		clusterVis.putAction(CLUSTER_GLYPH_COLOR, clusterColorList);

		createKClusterVis(kMeans, clusterTable);

	}

	public Display getDisplay() {
		return d;
	}

	public Display getClusterDisplay() {
		return clusterD;
	}

	public void setK(int k) {
		if (kMeans != null) {
			kMeans.cluster(k);
			recreateClusterDisplay();
		}
	}

	public void unfilterAll() {
		// check that we have at least 1 axis
		if (vis.getVisualGroup(COORDINATES) != null) {

			Iterator dataItems = vis.items(DATA);
			while (dataItems.hasNext()) {
				VisualItem item = (VisualItem) dataItems.next();
				item.setBoolean(IGNORE, false);
				fireEvent("ignore", item.getString(LABEL), "f");
			}
			selectAll();
			kMeans.recluster();
			recreateClusterDisplay();
		}

	}

	public void filterSelected() {
		if (vis.getVisualGroup(COORDINATES) != null) {

			TupleSet selected = vis.getFocusGroup(SELECTED);
			if (selected.getTupleCount() > 0) {
				Iterator selectedItems = selected.tuples();
				while (selectedItems.hasNext()) {
					VisualItem item = (VisualItem) selectedItems.next();
					item.setBoolean(IGNORE, true);
					fireEvent("ignore", item.getString(LABEL), "t");
				}
				selectAll();
				kMeans.recluster();
				recreateClusterDisplay();
			}
		}
	}

	public void alignAxes() {
		pcLayout.setUserControlled(false);
		vis.run("layout");
	}

	public void zoomAll() {
		yDisplay.zoomToFit(minData, maxData);
	}

	public void zoomToFit() {
		TupleSet selected = vis.getFocusGroup(SELECTED);
		if (selected.getTupleCount() > 0) {

			double selectMin = 100000000.0;
			double selectMax = -100000000.0;
			Iterator selectedItems = selected.tuples();
			while (selectedItems.hasNext()) {
				VisualItem item = (VisualItem) selectedItems.next();

				List values = (List) item.get(VALUES);
				Iterator valuesIt = values.iterator();
				while (valuesIt.hasNext()) {
					Double val = (Double) valuesIt.next();
					if (val.doubleValue() < selectMin) {
						selectMin = val.doubleValue();
					}
					if (val.doubleValue() > selectMax) {
						selectMax = val.doubleValue();
					}
				}
			}
			yDisplay.zoomToFit(selectMin, selectMax);
		} else {
			yDisplay.zoomToFit(minData, maxData);
		}
	}

	private void recreateClusterDisplay() {
		createKClusterVis(kMeans, clusterTable);
		clusterLayout(clusterD);

		/*
		 * double min = 100000000.0; double max = -100000000.0; Iterator
		 * dataItems = vis.items(DATA); while (dataItems.hasNext()) { VisualItem
		 * dataItem = (VisualItem) dataItems.next(); if
		 * (!dataItem.getBoolean(IGNORE)) { List values = (List)
		 * dataItem.get(VALUES); Iterator valuesIt = values.iterator(); while
		 * (valuesIt.hasNext()) { Double val = (Double) valuesIt.next(); if
		 * (val.doubleValue() < min) { min = val.doubleValue(); } if
		 * (val.doubleValue() > max) { max = val.doubleValue(); } } } }
		 * clusterGlyphRenderer.zoomToFit(min, max);
		 */
		clusterGlyphRenderer.zoomToFit(-1, 1);

	}

	private void clusterLayout(Display clusterD) {
		int y = clusterD.getHeight();
		int clustersPerColumn = (y / 40);
		if (clustersPerColumn < 1)
			clustersPerColumn = 1;
		int column = 0;
		int row = 0;
		TupleSet clusters = clusterVis.getVisualGroup(CLUSTERS);
		if (clusters != null) {
			Iterator clusterIt = clusters.tuples();
			int i = 0;
			while (clusterIt.hasNext()) {
				VisualItem item = (VisualItem) clusterIt.next();
				item.setX(column * 40);
				item.setY(row++ * 40);
				i++;
				if (clusterIt.hasNext()) {
					if (i % clustersPerColumn == 0) {
						column++;
						row = 0;
					}
				}
			}
			clusterD.repaint();
		}
	}

	public int getPreferredClusterSize(int height) {

		int clustersPerColumn = (height / 40);
		if (clustersPerColumn < 1)
			clustersPerColumn = 1;

		TupleSet clusters = clusterVis.getVisualGroup(CLUSTERS);
		if (clusters != null) {
			int clusterCount = clusters.getTupleCount();
			return ((clusterCount / clustersPerColumn) + 1) * 40;
		} else {
			return 40;
		}
	}

	private KMeansCluster createKClusterVis(KMeansCluster kMeans,
			Table clusterTable) {

		Iterator dataIt = d.getVisualization().getVisualGroup(DATA).tuples();
		while (dataIt.hasNext()) {
			VisualItem data = (VisualItem) dataIt.next();
			data.setStrokeColor(ColorLib.gray(0));
		}

		clusterTable.clear();
		for (int i = 0; i < kMeans.getK(); i++) {
			int id = clusterTable.addRow();
			clusterTable.set(id, "dataPoints", kMeans.getClusterCenter(i));
			clusterTable.setInt(id, "count", kMeans.getMembers(i).size());
		}
		Iterator clusterIt = clusterVis.getVisualGroup(CLUSTERS).tuples();
		int i = 0;
		while (clusterIt.hasNext()) {
			VisualItem item = (VisualItem) clusterIt.next();
			item.setStroke(StrokeLib.getStroke(1.0f));
			item.setX(0);
			item.setY(40 * i++);
			item.setSize(40);
		}
		clusterVis.run(CLUSTER_GLYPH_COLOR);
		return kMeans;
	}

	public void selectByFilters() {
		Set toSelect = new HashSet();
		Iterator items = vis.items(DATA);
		while (items.hasNext()) {
			VisualItem item = (VisualItem) items.next();
			if (!item.getBoolean(IGNORE) && item.isVisible()) {
				if (dataFilterer.passesFilters(item)) {
					toSelect.add(item);
				}
			}
		}

		setSelectedItems(toSelect);
	}

	public void selectAll() {
		Set toSelect = new HashSet();
		Iterator it = vis.items(DATA);
		while (it.hasNext()) {
			VisualItem item = (VisualItem) it.next();
			if (!item.getBoolean(IGNORE) && item.isVisible()) {
				toSelect.add(item);
			}
		}
		setSelectedItems(toSelect);
	}

	private giny.model.Node getCytoscapeNode(VisualItem vi) {
		int index = Cytoscape.getCyNode(vi.getString(LABEL))
				.getRootGraphIndex();
		return cyNetwork.getNode(index);
	}

	public void selectInCytoscape(Object[] visualItems, boolean selecting) {
		List cytoNodes = new ArrayList();
		for (int i = 0; i < visualItems.length; i++) {
			Object obj = visualItems[i];
			if (obj instanceof VisualItem) {
				VisualItem vi = (VisualItem) obj;
				cytoNodes.add(getCytoscapeNode(vi));
			} else {
				System.out.println("I don't know how to deal with this item.");
			}
		}
		if (cytoNodes.size() > 0) {
			cyNetwork.setSelectedNodeState(cytoNodes, selecting);
			Cytoscape.getCurrentNetworkView().redrawGraph(false, false);

		}
	}

	public void setSelectedItems(Set visualItems) {
		TupleSet curSelected = vis.getFocusGroup(SELECTED);
		List deselectNodes = new ArrayList();
		Iterator it = curSelected.tuples();
		while (it.hasNext()) {
			VisualItem item = (VisualItem) it.next();
			if (!visualItems.contains(item)) {
				deselectNodes.add(getCytoscapeNode(item));
			}
		}
		cyNetwork.setSelectedNodeState(deselectNodes, false);

		it = visualItems.iterator();
		List selectNodes = new ArrayList();
		while (it.hasNext()) {
			VisualItem item = (VisualItem) it.next();
			selectNodes.add(getCytoscapeNode(item));
		}
		cyNetwork.setSelectedNodeState(selectNodes, true);

		Cytoscape.getCurrentNetworkView().redrawGraph(false, false);

	}

	public void onSelectEvent(SelectEvent se) {
		// Cytoscape generated select event

		if (idToVisData != null) {
			TupleSet selected = vis.getFocusGroup(SELECTED);

			boolean selecting = se.getEventType();
			if (se.getTargetType() == SelectEvent.NODE_SET) {
				Collection cytoNodes = (Collection) se.getTarget();
				Iterator it = cytoNodes.iterator();
				while (it.hasNext()) {
					giny.model.Node cytoNode = (giny.model.Node) it.next();
					VisualItem item = (VisualItem) idToVisData.get(cytoNode
							.getIdentifier());
					if (item != null) {
						if (selecting) {
							selected.addTuple(item);
						} else {
							selected.removeTuple(item);
						}
					}
				}
			} else if (se.getTargetType() == SelectEvent.SINGLE_NODE) {
				giny.model.Node cytoNode = (giny.model.Node) se.getTarget();
				VisualItem item = (VisualItem) idToVisData.get(cytoNode
						.getIdentifier());
				if (item != null) {
					if (selecting) {
						selected.addTuple(item);
					} else {
						selected.removeTuple(item);
					}
				}
			}
			vis.run(INTERACTIVITY);
			vis.run(COLOR);
			clusterVis.run(ParallelCoordinates.CLUSTER_GLYPH_COLOR);
		}
	}

	private void setPCAxes(List attributeNames) {
		if (attributeNames != null && attributeNames.size() > 1) {
			List allValues = new ArrayList();
			List labels = new ArrayList();

			List attributeMaps = new ArrayList();
			for (int i = 0; i < attributeNames.size(); i++) {
				attributeMaps.add(CyAttributesUtils.getAttribute(
						(String) attributeNames.get(i), Cytoscape
								.getNodeAttributes()));
			}
			Iterator it = cyNetwork.nodesIterator();
			while (it.hasNext()) {
				giny.model.Node node = (giny.model.Node) it.next();
				List values = new ArrayList();
				for (int j = 0; j < attributeMaps.size(); j++) {
					Map attribMap = (Map) attributeMaps.get(j);
					Object attribVal = attribMap.get(node.getIdentifier());
					if (attribVal instanceof Double) {
						values.add(attribVal);
					} else if (attribVal instanceof String) {
						try {
							double newVal = Double
									.parseDouble((String) attribVal);
							values.add(new Double(newVal));
						} catch (NumberFormatException e) {
							values.add(new Double(0.0));
						}
					} else {
						values.add(new Double(0.0));
					}
				}

				labels.add(node.getIdentifier());
				allValues.add(values);
			}

			attributeChooser.setAttributeNames(attributeNames);
			loadData(attributeNames, allValues, labels);
			kMeans.recluster();
			recreateClusterDisplay();
			synchronizeSelectState();
		} else {

		}
	}

	private void synchronizeSelectState() {
		CyNetworkView cnv = Cytoscape.getNetworkView(cyNetwork.getIdentifier());
		if (cnv != null) {
			TupleSet selected = vis.getFocusGroup(SELECTED);
			selected.clear();
			Iterator nodesIt = cyNetwork.nodesIterator();
			while (nodesIt.hasNext()) {
				giny.model.Node cytoNode = (giny.model.Node) nodesIt.next();
				NodeView nodeView = cnv.getNodeView(cytoNode);
				if (nodeView != null) {
					if (nodeView.isSelected()) {
						VisualItem item = (VisualItem) idToVisData.get(cytoNode
								.getIdentifier());
						selected.addTuple(item);
					}
				}
			}
		}
		vis.run(INTERACTIVITY);
		vis.run(COLOR);
		clusterVis.run(ParallelCoordinates.CLUSTER_GLYPH_COLOR);
	}

	public void selectAttributes() {
		attributeChooser.showDialog();
		if (!attributeChooser.cancelled) {
			List attributeNames = attributeChooser.getAttributeNames();
			setNewConditions(attributeNames);
		}
	}

	public void setNewConditions(List attributeNames) {
		if (attributeNames.size() > 1) {
			setPCAxes(attributeNames);
			coordinateDragger.initializeAxisOrder();
		} else {
			d.removeAll();
			d.getVisualization().reset();
			// you'd think reset would remove all actions. Sadly no.
			// you can't make a null action, so just do a repaint.
			d.getVisualization().putAction("layout", new RepaintAction());
			clusterD.removeAll();
			clusterD.getVisualization().reset();
			d.repaint();
			clusterD.repaint();
		}
		saveAttributeSelection(attributeNames);

		Cerebral cerebralView = NetworksManager.instance().getCerebralView(
				cyNetwork.getIdentifier());
		if (cerebralView != null) {
			cerebralView.createExpressionViews();
		}

	}

	public void saveAttributeSelection(List attributeNames) {
		CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
		String saveString = "";
		for (int i = 0; i < attributeNames.size(); i++) {
			saveString += attributeNames.get(i);
			if (i < attributeNames.size() - 1) {
				saveString += DELIM;
			}
		}

		globalAttributes.setAttribute(Cytoscape.getCurrentNetwork()
				.getIdentifier(), "cerebralPCAttributes", saveString);
		NetworksManager.instance().setExpressionAttributes(
				cyNetwork.getIdentifier(), attributeNames);
	}

	public void loadSavedAxes() {
		setPCAxes(NetworksManager.instance().getExpressionAttributes(
				cyNetwork.getIdentifier()));
	}

	public void setQuality(boolean high) {
		d.setHighQuality(high);
		d.repaint();
	}

	public void fireEvent(String name, String old, String newVal) {
		firePropertyChange(name, old, newVal);
	}

	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName() == "quality") {
			Boolean q = (Boolean) e.getNewValue();
			d.setHighQuality(q.booleanValue());
			d.repaint();
		} else if (e.getPropertyName() == "ratioData") {
			d.invalidate();
			d.repaint();
		}
	}

	public void stateChanged(ChangeEvent e) {
		VisualStyle visualStyle = Cytoscape.getVisualMappingManager()
				.getVisualStyle();
		GlobalAppearanceCalculator gac = visualStyle
				.getGlobalAppearanceCalculator();
		GlobalAppearance ga = gac.calculateGlobalAppearance(cyNetwork);
		d.setBackground(ga.getBackgroundColor());
		clusterD.setBackground(ga.getBackgroundColor());
		border.setForeground(ga.getBackgroundColor());
		border.setBackground(ga.getBackgroundColor());
	}

	public void setHover(String nodeLabel, boolean b) {

		if (idToVisData != null) {
			VisualItem item = (VisualItem) idToVisData.get(nodeLabel);
			item.setHover(b);
			vis.run(COLOR);
		}
	}

	public void updateVisible(String nodeLabel, boolean visible) {
		if (idToVisData != null) {
			VisualItem item = (VisualItem) idToVisData.get(nodeLabel);
			item.setVisible(visible);
			vis.run(COLOR);
		}
	}

	public void axisMoved(int sourceLoc, int destLoc) {
		coordinateDragger.axisOrderChange(sourceLoc, destLoc);
	}

	public void setUserEditedCoordinates() {
		pcLayout.setUserControlled(true);

	}

	public void setSecondarySelectedAxis(String conditionName) {
		if (secondarySelection != null) {
			secondarySelection.setFillColor(ColorLib.color(Color.black));
			((VisualItem) secondarySelection.get(DECORATOR))
					.setTextColor(ColorLib.rgb(0, 0, 0));
			secondarySelection = null;
		}
		if (conditionName != null) {
			TupleSet coordinates = vis.getVisualGroup(COORDINATES);
			if (coordinates != null) {
				Iterator coordsIt = coordinates.tuples();
				while (coordsIt.hasNext()) {
					VisualItem item = (VisualItem) coordsIt.next();
					if (item.getString(LABEL).compareTo(conditionName) == 0) {
						secondarySelection = item;
						secondarySelection
								.setFillColor(ColorLib.rgb(0, 0, 128));
						((VisualItem) secondarySelection.get(DECORATOR))
								.setTextColor(ColorLib.rgb(0, 0, 128));

						break;
					}
				}
			}
		}

		vis.repaint();
	}

	public void setPrimarySelectedAxis(String conditionName) {

		if (primarySelection != null) {
			primarySelection.setFillColor(ColorLib.color(Color.black));
			((VisualItem) primarySelection.get(DECORATOR))
					.setTextColor(ColorLib.rgb(0, 0, 0));
			primarySelection = null;

		}
		if (conditionName != null) {
			TupleSet coordinates = vis.getVisualGroup(COORDINATES);
			if (coordinates != null) {
				Iterator coordsIt = coordinates.tuples();
				while (coordsIt.hasNext()) {
					VisualItem item = (VisualItem) coordsIt.next();
					if (item.getString(LABEL).compareTo(conditionName) == 0) {
						primarySelection = item;
						primarySelection.setFillColor(ColorLib
								.color(Color.blue));
						((VisualItem) primarySelection.get(DECORATOR))
								.setTextColor(ColorLib.color(Color.blue));

						break;
					}
				}
			}
		}
		vis.repaint();
	}

}
