package cerebral.parallelCoordinates.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.swing.SwingUtilities;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.controls.ControlAdapter;
import prefuse.data.tuple.TupleSet;
import prefuse.visual.VisualItem;
import cerebral.parallelCoordinates.CoordinateSorter;
import cerebral.parallelCoordinates.ParallelCoordinates;
import cerebral.parallelCoordinates.render.ClusterGlyphRenderer;
import cerebral.parallelCoordinates.render.DataRenderer;

public class CoordinateDragger extends ControlAdapter {
	private DataRenderer dataRenderer;

	private ClusterGlyphRenderer clusterGlyphRenderer;

	private Point2D down = new Point2D.Double();

	private Point2D temp = new Point2D.Double();


	private CoordinateSorter coordinateSorter = new CoordinateSorter();

	private VisualItem[] sorter;
	private VisualItem[] coordinateOrder;


	private Visualization vis;

	private ParallelCoordinates PC;
	
	private String prefix;

	public CoordinateDragger(ParallelCoordinates PC, Visualization vis,
			int dataDimensions, DataRenderer renderer,
			ClusterGlyphRenderer clusterGlyphRenderer,
			String prefix) {
		this.dataRenderer = renderer;
		this.clusterGlyphRenderer = clusterGlyphRenderer;
		this.vis = vis;
		this.PC = PC;
		this.prefix = prefix;

		coordinateOrder = new VisualItem[dataDimensions];
		sorter = new VisualItem[dataDimensions];
		initializeAxisOrder();

	}

	public void initializeAxisOrder() {
		TupleSet coordinates = vis
				.getVisualGroup(ParallelCoordinates.COORDINATES);

		int i = 0;
		Iterator coordsIt = coordinates.tuples();
		while (coordsIt.hasNext() && i < sorter.length) {
			VisualItem coordinate = (VisualItem) coordsIt.next();
			sorter[i++] = coordinate;
		}

		Arrays.sort(sorter, coordinateSorter);
		for (int j = 0; j < coordinateOrder.length; j++) {
			coordinateOrder[j] = sorter[j];
		}

	}

	public void itemEntered(VisualItem item, MouseEvent e) {
		if (item.isInGroup(ParallelCoordinates.COORDINATES)) {
			Display d = (Display) e.getSource();
			d.setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
		}

	}

	public void itemExited(VisualItem item, MouseEvent e) {
		if (item.isInGroup(ParallelCoordinates.COORDINATES)) {
			Display d = (Display) e.getSource();
			d.setCursor(Cursor.getDefaultCursor());
		}
	}

	public void itemPressed(VisualItem item, MouseEvent e) {
		if (item.isInGroup(ParallelCoordinates.COORDINATES)) {

			if (!SwingUtilities.isLeftMouseButton(e))
				return;
			Display d = (Display) e.getComponent();
			d.getAbsoluteCoordinate(e.getPoint(), down);
			PC.setUserEditedCoordinates();
		}
	}

	public void itemDragged(VisualItem item, MouseEvent e) {
		if (item.isInGroup(ParallelCoordinates.COORDINATES)) {
			if (!SwingUtilities.isLeftMouseButton(e))
				return;

			Display d = (Display) e.getComponent();
			d.getAbsoluteCoordinate(e.getPoint(), temp);
			double dx = temp.getX() - down.getX();
			double x = item.getX();
			if (x + dx < 40){
				dx = 40 - x;
			}else if (x + dx > d.getWidth()){
				dx = d.getWidth() - x;
			}
	
				moveAxis(item, x+dx);
							validateAxisOrder(item, dx > 0);

				dataRenderer.validate();

				item.getVisualization().repaint();

				down.setLocation(temp);
			
		}
	}

	
	private void moveAxis (VisualItem axis, double newX){
		axis.setX(newX);
		/*
		Set filters = (Set) axis.get(ParallelCoordinates.FILTERS);
		Iterator filtersIt = filters.iterator();
		while (filtersIt.hasNext()) {
			Interval filter = (Interval) filtersIt.next();
			VisualItem visFilter = filter.getVisualItem();
			visFilter.setX(newX);
		}
		*/
		vis.run("filterLayout");
		VisualItem label = (VisualItem) axis
				.get(ParallelCoordinates.DECORATOR);
		label.setX(newX);
	}
	
	private void validateAxisOrder(VisualItem axis, boolean right) {

		TupleSet coordinates = vis
				.getVisualGroup(ParallelCoordinates.COORDINATES);

		int i = 0;
		Iterator coordsIt = coordinates.tuples();
		while (coordsIt.hasNext() && i < sorter.length) {
			VisualItem coordinate = (VisualItem) coordsIt.next();
			sorter[i++] = coordinate;
		}

		Arrays.sort(sorter, coordinateSorter);

		int oldLocation = 0;
		for (int j = 0; j < coordinateOrder.length; j++) {
			if (coordinateOrder[j] == axis) {
				oldLocation = j;
				break;
			}
		}

		int newLocation = 0;
		for (int j = 0; j < coordinateOrder.length; j++) {
			if (sorter[j] == axis) {
				newLocation = j;
				break;
			}
		}

		if (oldLocation != newLocation) {
			for (int j = 0; j < coordinateOrder.length; j++) {
				coordinateOrder[j] = sorter[j];
			}

			clusterGlyphRenderer.validate();
			PC.fireEvent("axisReorder", prefix + axis
					.getString(ParallelCoordinates.LABEL), oldLocation + ":"
					+ newLocation);
			PC.saveAttributeSelection(getAttributeNamesAsList());
	        
		}

	}

	public void axisOrderChange(int sourceLoc, int destLoc) {
		double saveX = coordinateOrder[destLoc].getX();
		
		if (sourceLoc < destLoc){
			for (int i= destLoc; i > sourceLoc; i--){
				moveAxis(coordinateOrder[i], coordinateOrder[i-1].getX());
			}			
		}else{
			for (int i= destLoc; i < sourceLoc; i++){

				moveAxis(coordinateOrder[i], coordinateOrder[i+1].getX());
			}			
		}
		moveAxis(coordinateOrder[sourceLoc], saveX);
		initializeAxisOrder();

		
		PC.saveAttributeSelection(getAttributeNamesAsList());
		
		dataRenderer.validate();
		clusterGlyphRenderer.validate();
		vis.repaint();

	}
	
	private List getAttributeNamesAsList() {
	    List names = new ArrayList();
	    for (int i=0; i < coordinateOrder.length; i++){
	        VisualItem item = (VisualItem) coordinateOrder[i];
	        names.add (item.getString(ParallelCoordinates.LABEL));
	    }
        return names;
    }

    public void itemClicked(VisualItem item, MouseEvent e){
	    if (item.isInGroup(ParallelCoordinates.COORDINATE_LABELS)) {
            if (!SwingUtilities.isLeftMouseButton(e))
                return;
           
            String coordinateName = item.getString(ParallelCoordinates.LABEL);            
            PC.fireEvent("axisSelect", coordinateName,e.isShiftDown()?"y":"n");
	    }
            
        
	}

}
