package cerebral.parallelCoordinates.controls;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;
import java.util.Iterator;

import javax.swing.SwingUtilities;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.controls.ControlAdapter;
import prefuse.data.Table;
import prefuse.visual.VisualItem;
import cerebral.parallelCoordinates.YDisplay;
import cerebral.parallelCoordinates.ParallelCoordinates;

public class RubberBandSelect extends ControlAdapter {
    private int downX1, downY1;

    private VisualItem rubberBand;

    private Point2D screenPoint = new Point2D.Float();

    private Point2D absPoint = new Point2D.Float();

    private Rectangle2D rect = new Rectangle2D.Float();

    private Line2D line = new Line2D.Float();

    private ParallelCoordinates parallelCoordinates;
    private Table filterTable;
    private YDisplay axisDisplay;
    private NumberFormat nf = NumberFormat.getInstance();
    
    private boolean editing = false;
    private VisualItem editItem;
    private boolean editTop;
    
    public RubberBandSelect(ParallelCoordinates parallelCoordinates,
            YDisplay axisDisplay,
            Table filterTable,
        VisualItem rubberBand) {
        this.rubberBand = rubberBand;
        this.parallelCoordinates = parallelCoordinates;
        this.filterTable = filterTable;
        this.axisDisplay = axisDisplay;
        nf.setMaximumFractionDigits(2);
    }

    public void setRubberBand( VisualItem rubberBand){
        this.rubberBand = rubberBand;
    }
    
    public void itemPressed(VisualItem item, MouseEvent e) {
        if ( item.isInGroup(ParallelCoordinates.DATA)) {
            mousePressed(e);
        }
    }

    public void itemDragged(VisualItem item, MouseEvent e) {
        if ( item.isInGroup(ParallelCoordinates.DATA)) {
            mouseDragged(e);
        }
    }

    public void itemReleased(VisualItem item, MouseEvent e) {
        if ( item.isInGroup(ParallelCoordinates.DATA)) {
            mouseReleased(e);
        }
    }
    
    public void itemClicked(VisualItem item, MouseEvent e) {
        Display d = (Display)e.getComponent();
        if (editing) {         
            stopEditing(d);
        }
        if (SwingUtilities.isLeftMouseButton(e)) {
            if (item.isInGroup(ParallelCoordinates.FILTER_TOP_LABELS)){
                editTop = true;
                double curVal = item.getDouble(ParallelCoordinates.TOP);                
                item.setString(ParallelCoordinates.TEXT_EDIT, nf.format(curVal));
                startEditing(item, d);
            } else if (item.isInGroup(ParallelCoordinates.FILTER_BOTTOM_LABELS)){
                editTop = false;
                double curVal = item.getDouble(ParallelCoordinates.BOTTOM);                
                item.setString(ParallelCoordinates.TEXT_EDIT, nf.format(curVal));
                startEditing(item, d);
            } 
        }
    }

    public void mousePressed(MouseEvent e) {
        Display d = (Display) e.getComponent();
        
        if (editing) {
            stopEditing(d);
        }else{
        
            if (!SwingUtilities.isLeftMouseButton(e)||e.isAltDown())
                return;
    
            if (!e.isShiftDown()) {
                 filterTable.clear();            
                 parallelCoordinates.selectAll();             
            }
             
            screenPoint.setLocation(e.getX(), e.getY());
            d.getAbsoluteCoordinate(screenPoint, absPoint);
            downX1 = (int) absPoint.getX();
            downY1 = (int) absPoint.getY();
    
            rubberBand.setVisible(true);
        }

    }

    public void mouseDragged(MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e)||e.isAltDown())
                      return;

        Display d = (Display) e.getComponent();
        screenPoint.setLocation(e.getX(), e.getY());
        d.getAbsoluteCoordinate(screenPoint, absPoint);
        int x1 = downX1;
        int y1 = downY1;
        int x2 = (int) absPoint.getX();
        int y2 = (int) absPoint.getY();

        float[] bandRect = (float[]) rubberBand.get(VisualItem.POLYGON);
        bandRect[0] = x1;
        bandRect[1] = y1;
        bandRect[2] = x2;
        bandRect[3] = y1;
        bandRect[4] = x2;
        bandRect[5] = y2;
        bandRect[6] = x1;
        bandRect[7] = y2;
        rubberBand.setValidated(false);

        if (x2 < x1) {
            int temp = x2;
            x2 = x1;
            x1 = temp;
        }
        if (y2 < y1) {
            int temp = y2;
            y2 = y1;
            y1 = temp;
        }
        rect.setRect(x1, y1, x2 - x1, y2 - y1);

        Visualization vis = d.getVisualization();     

        Iterator it = vis.getGroup("coordinates").tuples();
        while (it.hasNext()) {
            VisualItem item = (VisualItem) it.next();
            line.setLine(item.getX(), -100000f, item.getX(), 100000f);
            
            int tempFilterId = item.getInt(ParallelCoordinates.TEMP_FILTER);
            if (tempFilterId >= 0){
                filterTable.removeRow(tempFilterId);
            }            
            if (line.intersects(rect)) {                
                double rangeBottom = axisDisplay.screenToValue(rect.getMaxY());
                double rangeTop = axisDisplay.screenToValue(rect.getMinY());
                tempFilterId = filterTable.addRow();
                item.setInt(ParallelCoordinates.TEMP_FILTER, tempFilterId);
                VisualItem visFilter = vis.getVisualItem(ParallelCoordinates.FILTERS, filterTable
                        .getTuple(tempFilterId));
                filterTable.setDouble(tempFilterId, ParallelCoordinates.TOP, rangeTop);
                filterTable.setDouble(tempFilterId, ParallelCoordinates.BOTTOM, rangeBottom);
                filterTable.set(tempFilterId, ParallelCoordinates.COORDINATE, item);

                visFilter.setX(item.getX());
                visFilter.setInteractive(false);
            }
        }
        parallelCoordinates.selectByFilters();
        d.getVisualization().repaint();
    }

    public void mouseReleased(MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e)||e.isAltDown())
                     return;
        rubberBand.setVisible(false);

        Display d = (Display) e.getComponent();   
        clearTempFilters(d);        
        d.getVisualization().repaint();
    }
    
    private void clearTempFilters(Display d){
        Visualization vis = d.getVisualization();
        Iterator it = vis.getGroup("coordinates").tuples();
        while (it.hasNext()) {
            VisualItem item = (VisualItem) it.next();
            item.setInt(ParallelCoordinates.TEMP_FILTER,-1);
        }
    }
    private void startEditing(VisualItem item, Display d){
        editing = true;
        d.editText(item, ParallelCoordinates.TEXT_EDIT);
        editItem = item;
        
    }
    
    private void stopEditing(Display d){
        editing = false;
        d.stopEditing();
        
        String newText = editItem.getString(ParallelCoordinates.TEXT_EDIT);
        try{
            double newVal = Double.parseDouble(newText);
            if (editTop){
                double bottomVal = editItem.getDouble(ParallelCoordinates.BOTTOM);
                if (newVal > bottomVal){
                    editItem.setDouble(ParallelCoordinates.TOP, newVal);
                }else{
                    editItem.setDouble(ParallelCoordinates.TOP, bottomVal);
                    editItem.setDouble(ParallelCoordinates.BOTTOM, newVal);                    
                }               
            }else{
                double topVal = editItem.getDouble(ParallelCoordinates.TOP);
                if (newVal < topVal){
                    editItem.setDouble(ParallelCoordinates.BOTTOM, newVal);
                }else{
                    editItem.setDouble(ParallelCoordinates.BOTTOM, topVal);
                    editItem.setDouble(ParallelCoordinates.TOP, newVal);                    
                }
                
            }
            parallelCoordinates.selectByFilters();
        }catch (NumberFormatException e){
        }
        
    }
    

    public void keyReleased(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER && editing) {
            Display d = (Display)e.getComponent();            
            stopEditing(d);
        }
    }
    public void itemKeyReleased(VisualItem item, KeyEvent e) {
        keyReleased(e);
    }
    
}
