package cerebral.parallelCoordinates.render;

import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.visual.VisualItem;
import cerebral.Cerebral;

public class RotatedTextRenderer implements Renderer {

    
    private double rotateDegrees;
    private AffineTransform at = new AffineTransform();
    public RotatedTextRenderer(int degrees) {
        rotateDegrees = Math.toRadians(degrees);

    }
    public boolean locatePoint(Point2D p, VisualItem item) {
        FontRenderContext frc = Renderer.DEFAULT_GRAPHICS.getFontRenderContext();
        String label = item.getString(Cerebral.LABEL);
        TextLayout textLayout = new TextLayout(label, FontLib.getFont("Verdana", 14), frc);
        
        at.setToIdentity();
        at.translate( item.getX(), item.getY());
        at.rotate (rotateDegrees);
        at.translate(-textLayout.getBounds().getWidth(),textLayout.getBounds().getHeight());
        Shape rotatedText = at.createTransformedShape(textLayout.getBounds());
        return rotatedText.contains(p);
       
       
    }

    public void render(Graphics2D g, VisualItem item) {
 
    	g.setColor(ColorLib.getColor(item.getTextColor()));      
    	FontRenderContext frc = Renderer.DEFAULT_GRAPHICS.getFontRenderContext();
        String label = item.getString(Cerebral.LABEL);
        TextLayout textLayout = new TextLayout(label, FontLib.getFont("Verdana", 14), frc);
        
        AffineTransform saveTransform = g.getTransform();
        at.setToIdentity();
        at.translate( item.getX(), item.getY());
        at.rotate (rotateDegrees);
        at.translate(-textLayout.getBounds().getWidth(),textLayout.getBounds().getHeight());
        
        g.setTransform(at);
        textLayout.draw( g, 0,0 );
        g.setTransform(saveTransform);
      
        
    }

    public void setBounds(VisualItem item) {
        FontRenderContext frc = Renderer.DEFAULT_GRAPHICS.getFontRenderContext();
        String label = item.getString(Cerebral.LABEL);
        TextLayout textLayout = new TextLayout(label, FontLib.getFont("Verdana", 14), frc);
        
        at.setToIdentity();
        at.translate( item.getX(), item.getY());
        at.rotate (rotateDegrees);
        at.translate(-textLayout.getBounds().getWidth(),textLayout.getBounds().getHeight());
        Shape rotatedText = at.createTransformedShape(textLayout.getBounds());
        item.setBounds (rotatedText.getBounds().getX(),rotatedText.getBounds().getY(),rotatedText.getBounds().getWidth(), rotatedText.getBounds().getHeight());
        
    }


}
